/**
 * BaumbachFormel.java
 * eu.gronos.kostenrechner (Kostenrechner)
 */
package eu.gronos.kostenrechner;

import java.util.ArrayList;

import javax.swing.JTable;

/**
 * Der Tenor f&uuml;r <b>Teilunterliegen bei Streitgenossenschaft</b> kann nach
 * der Baumbach&prime;schen Kostenformel erzeugt werden.
 * 
 * @author Peter Schuster (setrok)
 * @date 29.04.2014
 * 
 */
class BaumbachFormel implements UrteilsTenorierend, Tabulierend {

	private static final String BESCHREIBUNG = "Tenor bei Streitgenossen nach der sog. Baumbach'schen Formel (Hauptsachetenor, Kostentenor, Vollstreckbarkeitstenor sowie Streitwertbeschluss).";
	private static final String GRUENDE_91_92_100II = "Die Kostenentscheidung beruht auf §§ 91, 92, 100 Abs. 2 ZPO und der sogenannten Baumbach'schen Kostenformel,"
			+ " entsprechend der unterschiedlichen Beteiligung am Rechtsstreit: \n";
	private static final String KOPFZEILE_BEGINN = "Einzelangriffe / Verluste";
	private static final String ERGEBNIS_ZEILE = "Kostenquote hinsichtlich eines fiktiven Streitwerts von %,.2f EUR";
	private static final String VERLUST_GESAMT = "Verlust gesamt";
	private final int[] parteien;
	final BaumbachBeteiligtenListe baumbachBeteiligtenListe;
	private boolean b92ii;
	private boolean wirtschaftlichIdentisch;
	private double[][] kosten = null;
	private final StringBuilder gruende = new StringBuilder();
	private int[] range = new int[2];
	private ArrayList<Object[]> zeilenListe = null;
	private VollstreckbarkeitsListe vollstreckbarkeit;

	// gesamtschuldnerisch wird nicht gebraucht?
	// private final boolean[] gesamtschuldnerisch;

	/**
	 * Konstruktor:
	 * 
	 * @param baumbachBeteiligtenListe
	 *            ein Array BaumbachBeteiligter[] in der Reihenfolge KLAEGER,
	 *            DRITTWIDERBEKLAGTE (falls vorhanden) und BEKLAGTE, mit
	 *            mindestens einem KLAEGER und einem BEKLAGTEn.
	 * @param b92iiZpo
	 *            Soll § 92 II ZPO angewandt werden (verhältnismäßig geringe
	 *            Kostentragungen werden unterschlagen)
	 * @param wirtschaftlichIdentisch
	 *            ist eine etwaige Widerklage wirtschaftlich identisch mit der
	 *            Hauptklageforderung?
	 * @throws IllegalArgumentException
	 *             wird geworfen, wenn eine der Vorgaben nicht eingehalten
	 *             wurde.
	 */
	BaumbachFormel(BaumbachBeteiligtenListe baumbachBeteiligtenListe,
			boolean b92iiZpo, boolean wirtschaftlichIdentisch)
			throws IllegalArgumentException {
		super();
		if (baumbachBeteiligtenListe == null
				|| baumbachBeteiligtenListe.size() < 1)
			throw new IllegalArgumentException(
					UrteilsTenorierend.FEHLER_GAR_NIX_PARTEIEN);
		// if (gesamtschuldnerisch == null || gesamtschuldnerisch.length != 3)
		// throw new IllegalArgumentException(
		// "gesamtschuldnerisch darf nicht null sein und muss 3 Elemente haben!");
		parteien = new int[] { baumbachBeteiligtenListe.zaehleKlaeger(),
				baumbachBeteiligtenListe.zaehleDrittwiderbeklagte(),
				baumbachBeteiligtenListe.zaehleBeklagte() };
		if (parteien[0] < 1 || parteien[2] < 1)
			throw new IllegalArgumentException(
					UrteilsTenorierend.FEHLER_NIX_PARTEIEN);
		if (baumbachBeteiligtenListe
				.zaehleAnWiderklageBeteiligte(Beteiligter.BEKLAGTE) > 0
				&& (baumbachBeteiligtenListe
						.zaehleAnWiderklageBeteiligte(Beteiligter.KLAEGER) + baumbachBeteiligtenListe
						.zaehleAnWiderklageBeteiligte(Beteiligter.DRITTWIDERBEKLAGTE)) <= 0)
			throw new IllegalArgumentException(
					UrteilsTenorierend.FEHLER_NIX_WIDERBEKLAGTE);
		if (baumbachBeteiligtenListe
				.zaehleAnWiderklageBeteiligte(Beteiligter.BEKLAGTE) <= 0
				&& (baumbachBeteiligtenListe
						.zaehleAnWiderklageBeteiligte(Beteiligter.KLAEGER) + baumbachBeteiligtenListe
						.zaehleAnWiderklageBeteiligte(Beteiligter.DRITTWIDERBEKLAGTE)) > 0)
			throw new IllegalArgumentException(
					UrteilsTenorierend.FEHLER_NIX_WIDERKLAEGER);
		// TODO (P) wenn ein BaumbachGesamtschuldner auf Bk-Seite
		// widerklagebeteiligt ist!
		if (baumbachBeteiligtenListe.zaehleKlaeger() > 1
				|| baumbachBeteiligtenListe
						.zaehleAnWiderklageBeteiligte(Beteiligter.BEKLAGTE) > 1)
			throw new IllegalArgumentException(
					UrteilsTenorierend.FEHLER_NUR_EINER);
		if (!baumbachBeteiligtenListe.istReihenfolgeEingehalten())
			throw new IllegalArgumentException(
					UrteilsTenorierend.FEHLER_REIHENFOLGE_BETEILIGTE);
		this.baumbachBeteiligtenListe = baumbachBeteiligtenListe;
		this.b92ii = b92iiZpo;
		// this.gesamtschuldnerisch = gesamtschuldnerisch;
		this.wirtschaftlichIdentisch = wirtschaftlichIdentisch;
	}

	/**
	 * Berechnet die Verteilung der Gerichtskosten
	 * 
	 * @return ein eindimensionales Array double[] mit einer Spalte für jeden
	 *         Beteiligten
	 */
	private double[] ermittleGerichtskostenQuoten() {
		double[] zwischen;
		double fiktiverStreitwert;
		zwischen = new double[baumbachBeteiligtenListe.size()];
		// fiktiven Streitwert berechnen
		fiktiverStreitwert = baumbachBeteiligtenListe.addiereStreitwerte();
		// Unterliegen des Klägers: Anderer fiktiver Streitwert nötig: nur die
		// Summe der Bk-Streitwerte!
		double klaegerUnterliegen = baumbachBeteiligtenListe
				.addiereStreitwerte(Beteiligter.BEKLAGTE)
				- baumbachBeteiligtenListe
						.addiereUnterliegen(Beteiligter.BEKLAGTE);
		// Bei Widerklage: zusätzlich noch das eigene Unterliegen des Klägers
		// hinsichtlich Widerklage
		if (baumbachBeteiligtenListe.isAnWiderklageBeteiligt(0))
			klaegerUnterliegen += baumbachBeteiligtenListe.getUnterliegenFor(0,
					false, false);
		zwischen[0] = (klaegerUnterliegen) * 100.0 / fiktiverStreitwert;
		// Wenn DWBk vorhanden: Unterliegen jedes DwBk berechnen
		if (baumbachBeteiligtenListe.zaehleDrittwiderbeklagte() > 0) {
			int[] dwbkBereich = baumbachBeteiligtenListe
					.minUndMax(Beteiligter.DRITTWIDERBEKLAGTE);
			for (int i = dwbkBereich[0]; i <= dwbkBereich[1]; i++) {
				double dwbkUnterliegen = 0.0;
				// Dwbk können BaumbachGesamtschuldner sein, deshalb:
				// beiGesamtschuldnernMultiplizieren = true
				dwbkUnterliegen += baumbachBeteiligtenListe.getUnterliegenFor(i,
						true, false);
				zwischen[i] = dwbkUnterliegen * 100.0 / fiktiverStreitwert;
			}
		}
		// Unterliegen jedes Beklagten
		int[] beklagtenBereich = baumbachBeteiligtenListe
				.minUndMax(Beteiligter.BEKLAGTE);
		for (int i = beklagtenBereich[0]; i <= beklagtenBereich[1]; i++) {
			// Erst einmal das eigene Unterliegen hinsichtlich der Klage
			double bkUnterliegen = baumbachBeteiligtenListe.getUnterliegenFor(i,
					true, false);
			// Um Unterliegen bzgl. Widerklage bei jedem Widerbeklagten erhöhen,
			// sofern auch Widerkläger
			if (baumbachBeteiligtenListe.isAnWiderklageBeteiligt(i)
					&& !baumbachBeteiligtenListe.isGesamtschuldnerschaft(i))
				bkUnterliegen += baumbachBeteiligtenListe
						.addiereStreitwerte(Beteiligter.KLAEGER)
						- baumbachBeteiligtenListe
								.addiereUnterliegen(Beteiligter.KLAEGER)
						+ baumbachBeteiligtenListe
								.addiereStreitwerte(Beteiligter.DRITTWIDERBEKLAGTE)
						- baumbachBeteiligtenListe
								.addiereUnterliegen(Beteiligter.DRITTWIDERBEKLAGTE);
			zwischen[i] = bkUnterliegen * 100.0 / fiktiverStreitwert;
		}
		baueAngriffsTabelle(zwischen, fiktiverStreitwert);
		baueGruende();
		return zwischen;
	}

	/**
	 * Berechnet die Verteilung der außergerichtlichen Kosten des Klägers.
	 * 
	 * @return ein zweidimensionales Array double[][] mit einer Zeile je Kläger
	 *         und einer Spalte für jeden Beteiligten.
	 */
	private double[][] ermittleKlaegerkostenQuoten() {
		double[][] zwischen;
		double fiktiverStreitwert;
		// TODO Geht so nur für einen (!) Kläger
		final int kl = 0;
		zwischen = new double[1][baumbachBeteiligtenListe.size()];
		// fiktiven Streitwert berechnen
		// der fiktive Streitwert ist die Summe aller Bk-Streitwerte
		// sowie des kl.eigenen Sw
		fiktiverStreitwert = baumbachBeteiligtenListe.getStreitwertFor(kl)
				+ baumbachBeteiligtenListe
						.addiereStreitwerte(Beteiligter.BEKLAGTE);
		// Unterliegen des Klägers: Anderer fiktiver Streitwert nötig: nur die
		// Summe der Bk-Streitwerte!
		double klUnterliegen = baumbachBeteiligtenListe
				.addiereStreitwerte(Beteiligter.BEKLAGTE)
				- baumbachBeteiligtenListe
						.addiereUnterliegen(Beteiligter.BEKLAGTE);
		// Bei Widerklage muss hier noch das eigene Unterliegen hinzugerechnet
		// werden.
		if (baumbachBeteiligtenListe.isAnWiderklageBeteiligt(kl))
			klUnterliegen += baumbachBeteiligtenListe.getUnterliegenFor(kl, false,
					true);
		zwischen[kl][kl] = klUnterliegen * 100 / fiktiverStreitwert;
		int[] beklagtenBereich = baumbachBeteiligtenListe
				.minUndMax(Beteiligter.BEKLAGTE);
		for (int i = beklagtenBereich[0]; i <= beklagtenBereich[1]; i++) {
			// Inwieweit war der jeweilige Beklagte hinsichtlich
			// der Klage unterlegen?
			double bkUnterliegen = baumbachBeteiligtenListe.getUnterliegenFor(i,
					true, false);
			// Bei Widerklage muss hier noch mehr hinzugerechnet werden:
			// Inwieweit war der Beklagte (sofern Widerkläger und nicht
			// Gesamtschuldnervereinigung) mit seiner Widerklage unterlegen?
			if (baumbachBeteiligtenListe.isAnWiderklageBeteiligt(i)
					&& baumbachBeteiligtenListe.isAnWiderklageBeteiligt(kl)
					&& !baumbachBeteiligtenListe.isGesamtschuldnerschaft(i))
				bkUnterliegen += (baumbachBeteiligtenListe.getStreitwertFor(kl) - baumbachBeteiligtenListe
						.getUnterliegenFor(kl, false, true));
			zwischen[kl][i] = (bkUnterliegen / fiktiverStreitwert) * 100.0;
		}
		return zwischen;
	}

	/**
	 * Berechnet die Verteilung der außergerichtlichen Kosten des
	 * Drittwiderbeklagten
	 * 
	 * @return ein zweidimensionales Array double[][] mit einer Zeile je
	 *         Drittwiderbeklagten und einer Spalte für jeden Beteiligten
	 */
	private double[][] ermittleDrittwiderbeklagtenkostenQuoten() {
		if (baumbachBeteiligtenListe.zaehleDrittwiderbeklagte() < 1)
			return null;
		double[][] zwischen = new double[baumbachBeteiligtenListe
				.zaehleDrittwiderbeklagte()][baumbachBeteiligtenListe.size()];
		final int[] dwbkBereich = baumbachBeteiligtenListe
				.minUndMax(Beteiligter.DRITTWIDERBEKLAGTE);
		final int[] bkBereich = baumbachBeteiligtenListe
				.minUndMax(Beteiligter.BEKLAGTE);
		for (int zeile = 0; zeile < baumbachBeteiligtenListe
				.zaehleDrittwiderbeklagte(); zeile++) {
			for (int beteiligter = 0; beteiligter < baumbachBeteiligtenListe
					.size(); beteiligter++) {
				// Der Dwbk, dem die zeile "gehört"
				if (beteiligter == zeile + dwbkBereich[0]
						&& !baumbachBeteiligtenListe
								.isGesamtschuldnerschaft(zeile + dwbkBereich[0]))
					zwischen[zeile][beteiligter] = baumbachBeteiligtenListe
							.getUnterliegenFor(beteiligter, false, true)
							* 100.0
							/ baumbachBeteiligtenListe
									.getStreitwertFor(beteiligter);
				// TODO: das klappt so nur, wenn es nur einen
				// widerklagenden Beklagten gibt.
				else if (beteiligter >= bkBereich[0]
						&& baumbachBeteiligtenListe
								.isAnWiderklageBeteiligt(beteiligter)
						&& !baumbachBeteiligtenListe
								.isGesamtschuldnerschaft(zeile + dwbkBereich[0])) {
					// Wenn es die Spalte des widerklagenden Beklagten ist
					zwischen[zeile][beteiligter] = (baumbachBeteiligtenListe
							.getStreitwertFor(zeile + dwbkBereich[0]) - baumbachBeteiligtenListe
							.getUnterliegenFor(zeile + dwbkBereich[0], false, true))
							* 100.0
							/ baumbachBeteiligtenListe.getStreitwertFor(zeile
									+ dwbkBereich[0]);
				} else
					// Rest mit 0.0 auffüllen
					zwischen[zeile][beteiligter] = 0.0;
			}
		}
		return zwischen;
	}

	/**
	 * Berechnet die Verteilung der außergerichtlichen Kosten der Beklagten
	 * 
	 * @return Array double[][] mit so vielen Zeilen wie Beklagte und der
	 *         Einfachheit halber (Einheitlichkeit) so vielen Spalten wie
	 *         Parteien insgesamt.
	 */
	private double[][] ermittleBeklagtenkostenQuoten() {
		double[][] zwischen = null;
		// Array muss so viele Zeilen wie Beklagte und der Einfachheit halber
		// (Einheitlichkeit) so viele Spalten wie Parteien insgesamt haben.
		zwischen = new double[baumbachBeteiligtenListe.zaehleBeklagte()][baumbachBeteiligtenListe
				.size()];
		final int[] bkBereich = baumbachBeteiligtenListe
				.minUndMax(Beteiligter.BEKLAGTE);
		final int[] klDwbkBereich = new int[] {
				baumbachBeteiligtenListe.minUndMax(Beteiligter.KLAEGER)[0],
				baumbachBeteiligtenListe
						.minUndMax(Beteiligter.DRITTWIDERBEKLAGTE)[1] };
		// zeilen-Schleife (i): für jeden Beklagten eine Zeile
		for (int zeile = 0; zeile < baumbachBeteiligtenListe.zaehleBeklagte(); zeile++) {
			// int betroffenerBeklagter: Index des betroffenen Beklagten
			final int betroffenerBeklagter = zeile + bkBereich[0];
			// Fiktiven Streitwert für den betroffenen Beklagten berechnen
			double fiktiverStreitwert = baumbachBeteiligtenListe
					.getStreitwertFor(betroffenerBeklagter);
			/*
			 * Das Unterliegen des Beklagten selbst setzt sich zusammen aus
			 * seinem Unterliegen hinsichtlich der Klage (und ggfls. der Wk)
			 * Zusätzliches Unterliegen wird einberechnet
			 */
			double bkUnterliegen = baumbachBeteiligtenListe.getUnterliegenFor(
					betroffenerBeklagter, false, true);
			/*
			 * Wenn er Widerkläger ist: fiktiven Streitwert um Streitwert von
			 * jedem widerbeklagten Kl./Dwbk erhöhen
			 */
			if (baumbachBeteiligtenListe
					.isAnWiderklageBeteiligt(betroffenerBeklagter))
				for (int klDwbk = klDwbkBereich[0]; klDwbk <= klDwbkBereich[1]; klDwbk++)
					if (baumbachBeteiligtenListe
							.isAnWiderklageBeteiligt(klDwbk))
						fiktiverStreitwert += baumbachBeteiligtenListe
								.getStreitwertFor(klDwbk);
			/*
			 * Spalten-Schleife int beteiligter: Für jede Partei eine Spalte
			 */
			for (int beteiligter = 0; beteiligter <= bkBereich[1]; beteiligter++) {
				/*
				 * Wenn der betroffeneBeklagte ein BaumbachGesamtschuldner ist,
				 * wird er übersprungen (sonst: Division durch 0)
				 */
				if (beteiligter >= klDwbkBereich[0]
						&& beteiligter <= klDwbkBereich[1]
						&& !baumbachBeteiligtenListe
								.isGesamtschuldnerschaft(betroffenerBeklagter)) {
					/*
					 * Hier wird's bei Gesamtschuldnerischer
					 * Widerklage-Verurteilung kritisch: Bei
					 * Gesamtschuldnerschaft muss multipliziert werden (deshalb
					 * true), sonst trägt eine Kl/Dwbk-Gesamtschuldnerschaft zu
					 * wenig. Aber das eigene Unterliegen des Klägers darf nicht
					 * hinzugerechnet werden, wenn er selbst an der Reihe ist,
					 * sonst trägt er zuviel; die Kl/Dwbk-Gesamtschuldnerschaft
					 * kommt ja auch noch an die Reihe. Deshalb dort false.
					 */
					final double klUnterliegen = baumbachBeteiligtenListe
							.getUnterliegenFor(beteiligter, true, false);
					/*
					 * Kläger: Zusätzliches Unterliegen wird einberechnet. Hier
					 * braucht man auch das Unterliegen des Beklagten
					 * hinsichtlich einer gesamtschuldnerischen Verurteilung,
					 * weil die Bk-Gesamtschuldnerschaft mit den
					 * Bk-Außergerichtskosten nichts zu schaffen hat.
					 */
					if (baumbachBeteiligtenListe.getTypFor(beteiligter) == Beteiligter.KLAEGER)
						zwischen[zeile][beteiligter] = (baumbachBeteiligtenListe
								.getStreitwertFor(betroffenerBeklagter) - (baumbachBeteiligtenListe
								.getUnterliegenFor(betroffenerBeklagter, false,
										true)))
								* 100 / fiktiverStreitwert;
					else
						/*
						 * den Dwbk geht das Unterliegen des Bk hinsichtlich der
						 * Klage nichts an
						 */
						zwischen[zeile][beteiligter] = 0.0;
					/*
					 * Wenn Bk und Kl/Dwbk an Wk beteiligt sind, wird das
					 * bkUnterliegen heraufgesetzt...
					 * 
					 * (hier muss bei Gesamtschuldnerschaften multiplziert
					 * werden, sobald diese an der Reihe ist. Bei den einfachen
					 * Beteiligten darf aber nicht deren Unterliegen in der
					 * Gesamtschuldnerschaft hinzugerechnet werden (die kommen
					 * ja auch noch dran, da bkUnterliegen über die
					 * Spalten-Schleife hinweg fortlebt)
					 */
					if ((baumbachBeteiligtenListe
							.isAnWiderklageBeteiligt(betroffenerBeklagter) && (baumbachBeteiligtenListe
							.isAnWiderklageBeteiligt(beteiligter)))) {
						bkUnterliegen += (baumbachBeteiligtenListe
								.getStreitwertFor(beteiligter) - baumbachBeteiligtenListe
								.getUnterliegenFor(beteiligter, true, false));
						/*
						 * ...und beim Kl./Dwbk wird das Unterliegen
						 * hinsichtlich der Wk hinzugerechnet
						 */
						zwischen[zeile][beteiligter] += (klUnterliegen * 100.0 / fiktiverStreitwert);
					}
				} else if (beteiligter == betroffenerBeklagter
						&& !baumbachBeteiligtenListe
								.isGesamtschuldnerschaft(betroffenerBeklagter)) {
					// Der betreffende Beklagte, dem die Zeile "gehört"
					zwischen[zeile][beteiligter] = bkUnterliegen;
					zwischen[zeile][beteiligter] *= 100.0;
					zwischen[zeile][beteiligter] /= fiktiverStreitwert;
				} else {
					/*
					 * Rest mit Nullen auffüllen (Beklagte müssen untereinander
					 * nix tragen)
					 */
					zwischen[zeile][beteiligter] = 0.0;
				}
			}
		}
		return zwischen;
	}

	/**
	 * Berechnet alle Kostenquoten, die gerichtlichen und die außergerichtlichen
	 * aller Beteiligten.
	 * 
	 * @return ein zweidimensionales Array double[][] mit einer Zeile für die
	 *         Gerichtskosten, und je einer weiteren Zeile für jeden
	 *         Beteiligten. In jeder Zeile hat jeder Beteiligte eine Spalte.
	 */
	private double[][] ermittleQuoten() {
		int beteiligte = baumbachBeteiligtenListe.size();
		double[][] zwischen = new double[beteiligte + 1][beteiligte];
		int zaehler = 0;
		zwischen[zaehler] = ermittleGerichtskostenQuoten();
		for (double[] quoten : ermittleKlaegerkostenQuoten()) {
			zaehler++;
			zwischen[zaehler] = quoten;
		}
		if (baumbachBeteiligtenListe.zaehleDrittwiderbeklagte() > 0)
			for (double[] quoten : ermittleDrittwiderbeklagtenkostenQuoten()) {
				zaehler++;
				zwischen[zaehler] = quoten;
			}
		for (double[] quoten : ermittleBeklagtenkostenQuoten()) {
			zaehler++;
			zwischen[zaehler] = quoten;
		}
		return zwischen;
	}

	/**
	 * Wenn eine der Quoten unter 10% ist, muss sie den verbleibenden sinnvoll
	 * "aufgeschlagen" werden:
	 * 
	 * § 92 Abs. 2 ZPO: Das Gericht kann der einen Partei die gesamten
	 * Prozesskosten auferlegen, wenn 1. die Zuvielforderung der anderen Partei
	 * verhältnismäßig geringfügig war und keine oder nur geringfügig höhere
	 * Kosten veranlasst hat oder 2. der Betrag der Forderung der anderen Partei
	 * von der Festsetzung durch richterliches Ermessen, von der Ermittlung
	 * durch Sachverständige oder von einer gegenseitigen Berechnung abhängig
	 * war.
	 * 
	 * @param quoten
	 *            ein Array double[] mit den quoten im Urzustand
	 * @return ein Array double[] mit den quoten unter Berücksichtigung von § 92
	 *         Abs. 2 ZPO
	 */
	private double[] entferneUnwesentlichesUnterliegen(double[] quoten) {
		double[] bereinigteQuoten = new double[quoten.length];
		double summe = 0.0;
		// Erster Schritt: alle Quoten durchgehen
		for (int index = 0; index < bereinigteQuoten.length; index++) {
			// ... und Einträge unter 10% ("verhältnismäßig gering") nicht
			// übernehmen ...
			if (quoten[index] < 10.0)
				bereinigteQuoten[index] = 0.0;
			else {
				// / ... andere hingegen aufsummieren
				bereinigteQuoten[index] = quoten[index];
				summe += quoten[index];
			}
		}
		// Wenn es keine Einzelquoten über 10% gab, dann das Array unversehrt
		// zurückgeben; ebenso, wenn die summe schon über 100% geht
		if (summe <= 0.0 || summe > 100.0)
			return quoten;
		// Zweiter Schritt: Sonst das Verhältnis der Summe zu 100% feststellen
		final double verhaeltnis = 100.0 / summe;
		// Dritter Schritt: Alle Quoten des neuen Arrays mit dem verhaeltnis
		// multiplizieren
		for (int index = 0; index < bereinigteQuoten.length; index++) {
			bereinigteQuoten[index] = bereinigteQuoten[index] * verhaeltnis;
		}
		return bereinigteQuoten;
	}

	/**
	 * Die Methode erzeugt den Hauptsachetenor, davon ausgehend, dass die
	 * Verurteilungen auf Zahlung lauten.
	 * 
	 * @return den Hauptsachetenor als String mit abschließendem Zeilenumbruch
	 * @see eu.gronos.kostenrechner.UrteilsTenorierend#erzeugeHauptsachetenor()
	 */
	@Override
	public String erzeugeHauptsachetenor() {
		/**
		 * Die lokale Variable speichert, ob Klage (Zeile 0) bzw. Widerklage
		 * (Zeile 1) ganz zugesprochen (Spalte 0) oder ganz abgewiesen
		 * (Spalte 1) wurden. Ansonsten muss ein
		 * "Im Übrigen wird die (Wider-)Klage abgewiesen" an den Tenor gehängt
		 * werden. Die Initialisierung geht davon aus, dass gar nicht verurteilt
		 * wurde, solange keine Verurteilung festgestellt ist.
		 */
		boolean[][] ganzZugesprochenOderAbgewiesen = new boolean[][] {
				{ false, true }, { false, true } };
		String[] tenorSaetze = new String[baumbachBeteiligtenListe.size()];
		// new String[baumbachBeteiligtenListe.zaehleBeklagte()];
		// String[] widerKlageTenorSaetze = null;
		/* erst die Verurteilungen nach der Klage */
		int[] bkBereich = baumbachBeteiligtenListe
				.minUndMax(Beteiligter.BEKLAGTE);
		int widerKlaegerIndex = -1;
		for (int index = bkBereich[0]; index <= bkBereich[1]; index++) {
			int kl = 0;
			if (baumbachBeteiligtenListe.isAnWiderklageBeteiligt(index))
				widerKlaegerIndex = index;
			if (baumbachBeteiligtenListe.getUnterliegenFor(index, false, false) > 0) {
				/*
				 * Nach darüber hinaus gucken, um es an der richtigen Stelle in
				 * den Tenor zu stecken.
				 */
				String darueberHinaus = baumbachBeteiligtenListe
						.hatDarueberHinaus(index) ? UrteilsTenorierend.DARUEBER_HINAUS
						: "";
				tenorSaetze[index] = String
						.format("%s %s verurteilt, %,.2f EUR an %s zu zahlen.",
								TenorUtils.satzBeginn(baumbachBeteiligtenListe
										.parteiBezeichner(
												Beteiligter.NOMINATIV, index)),
								TenorUtils
										.getWirdWerden(baumbachBeteiligtenListe
												.get(index).isPlural())
										+ darueberHinaus,
								baumbachBeteiligtenListe.getUnterliegenFor(index,
										false, false), baumbachBeteiligtenListe
										.parteiBezeichner(
												Beteiligter.AKKUSATIV, kl));
				/*
				 * Sobald eine Verurteilung vorliegt, wurde die Klage jedenfalls
				 * nicht mehr ganz abgewiesen.
				 */
				ganzZugesprochenOderAbgewiesen[0][1] = false;
			} else
				tenorSaetze[index] = null;
		}
		/*
		 * Wenn die Summe der Streitwerte so hoch ist wie die Summe des
		 * Unterliegens, dann hat der Kläger voll obsiegt.
		 */
		ganzZugesprochenOderAbgewiesen[0][0] = baumbachBeteiligtenListe
				.addiereStreitwerte(Beteiligter.BEKLAGTE) <= baumbachBeteiligtenListe
				.addiereUnterliegen(Beteiligter.BEKLAGTE);
		// dann die Verurteilungen nach der Widerklage
		if (baumbachBeteiligtenListe.enthaeltAnWiderklageBeteiligte()) {
			int[] klDwbkBereich = new int[] {
					baumbachBeteiligtenListe.minUndMax(Beteiligter.KLAEGER)[0],
					baumbachBeteiligtenListe
							.minUndMax(Beteiligter.DRITTWIDERBEKLAGTE)[1] };
			for (int index = klDwbkBereich[0]; index <= klDwbkBereich[1]; index++)
				if (baumbachBeteiligtenListe
						.getUnterliegenFor(index, false, false) > 0) {
					String darueberHinaus = baumbachBeteiligtenListe
							.hatDarueberHinaus(index) ? UrteilsTenorierend.DARUEBER_HINAUS
							: "";
					tenorSaetze[index] = String
							.format("Auf die Widerklage %s %s verurteilt, %,.2f EUR an %s zu zahlen.",
									TenorUtils
											.getWirdWerden(baumbachBeteiligtenListe
													.get(index).isPlural())
											+ darueberHinaus,
									baumbachBeteiligtenListe.parteiBezeichner(
											Beteiligter.NOMINATIV, index),
									baumbachBeteiligtenListe.getUnterliegenFor(
											index, false, false),
									baumbachBeteiligtenListe.parteiBezeichner(
											Beteiligter.AKKUSATIV,
											widerKlaegerIndex));
					/*
					 * Sobald eine Verurteilung vorliegt, wurde die Widerklage
					 * jedenfalls nicht mehr ganz abgewiesen.
					 */
					ganzZugesprochenOderAbgewiesen[1][1] = false;
				} else
					tenorSaetze[index] = null;
		}
		/*
		 * Ermitteln, an welche Stelle des Tenors die Abweisung im Übrigen
		 * gehört. Jeweils an die Stelle, an der der letzte Beklagte (bzw.
		 * Widerklage) erwähnt wird; und wo das ist, das sagt mir
		 * findeGesamtschuldnerIndices
		 */
		// Erst für die Klage
		ArrayList<Integer> fgsi = baumbachBeteiligtenListe
				.findeGesamtschuldnerIndices(Beteiligter.BEKLAGTE, false);
		int letzterBeklagter = 0;
		int letzterWiderbeklagter = 0;
		if (fgsi != null && fgsi.size() > 0)
			letzterBeklagter = fgsi.get(fgsi.size() - 1);
		// Dann für die Widerklage (zwei Schritte)
		fgsi = baumbachBeteiligtenListe.findeGesamtschuldnerIndices(
				Beteiligter.KLAEGER, false);
		if (fgsi != null && fgsi.size() > 0) {
			letzterWiderbeklagter = fgsi.get(fgsi.size() - 1);
			if (!baumbachBeteiligtenListe
					.isAnWiderklageBeteiligt(letzterWiderbeklagter))
				letzterWiderbeklagter = 0;
		}
		fgsi = baumbachBeteiligtenListe.findeGesamtschuldnerIndices(
				Beteiligter.DRITTWIDERBEKLAGTE, false);
		if (fgsi != null && fgsi.size() > 0)
			letzterWiderbeklagter = fgsi.get(fgsi.size() - 1);
		/*
		 * Wenn die Summe der Streitwerte so hoch ist wie die Summe des
		 * Unterliegens, dann hat der Widerkläger voll obsiegt.
		 */
		ganzZugesprochenOderAbgewiesen[1][0] = (baumbachBeteiligtenListe
				.addiereStreitwerte(Beteiligter.KLAEGER) + baumbachBeteiligtenListe
				.addiereStreitwerte(Beteiligter.DRITTWIDERBEKLAGTE)) <= (baumbachBeteiligtenListe
				.addiereUnterliegen(Beteiligter.KLAEGER) + baumbachBeteiligtenListe
				.addiereUnterliegen(Beteiligter.DRITTWIDERBEKLAGTE));
		String tenor = "";
		// Gesamtschuldner zuerst, Klagetenor vor Widerklagetenor.
		for (int index : baumbachBeteiligtenListe
				.beteiligteInTenorReihenfolge(false)) {
			if (tenorSaetze[index] != null)
				tenor += TenorUtils
						.verbessereGesamtschuldner(tenorSaetze[index]) + "\n";
			// Zusatz für Voll- oder Teilabweisung der Klage
			if (index == letzterBeklagter)
				if (ganzZugesprochenOderAbgewiesen[0][1])
					tenor += "Die Klage wird abgewiesen.\n";
				else if (!ganzZugesprochenOderAbgewiesen[0][0])
					tenor += "Im Übrigen wird die Klage abgewiesen.\n";
			// Zusatz für Voll- oder Teilabweisung der Widerklage
			if (index == letzterWiderbeklagter
					&& baumbachBeteiligtenListe
							.enthaeltAnWiderklageBeteiligte())
				if (ganzZugesprochenOderAbgewiesen[1][1])
					tenor += "Die Widerklage wird abgewiesen.\n";
				else if (!ganzZugesprochenOderAbgewiesen[1][0])
					tenor += "Im Übrigen wird die Widerklage abgewiesen.\n";
		}
		// if (ganzZugesprochenOderAbgewiesen[0][1]
		// || (baumbachBeteiligtenListe.enthaeltAnWiderklageBeteiligte() &&
		// ganzZugesprochenOderAbgewiesen[1][1])) {
		// tenor = ganzZugesprochenOderAbgewiesen[0][1] ? tenor
		// + "Die Klage wird abgewiesen.\n" : tenor;
		// tenor = baumbachBeteiligtenListe.enthaeltAnWiderklageBeteiligte()
		// && ganzZugesprochenOderAbgewiesen[1][1] ? tenor
		// + "Die Widerklage wird abgewiesen.\n" : tenor;
		// }
		// else {
		// if (!ganzZugesprochenOderAbgewiesen[0][0])
		// tenor += "Im Übrigen wird die Klage abgewiesen.\n";
		// if (baumbachBeteiligtenListe.enthaeltAnWiderklageBeteiligte()
		// && !ganzZugesprochenOderAbgewiesen[1][0])
		// tenor += "Im Übrigen wird die Widerklage abgewiesen.\n";
		// }
		return tenor;
	}

	/**
	 * Findet über die BaumbachBeteiligtenListe alle Gesamtschuldnerschaften, in
	 * denen der BaumbachBeteiligte mit dem Index beteiligter ist, dann gucken,
	 * ob sie > 0% tragen müssen
	 * 
	 * @param index
	 *            die Position in der BaumbachBeteiligtenListe
	 * @param spalten
	 *            ein Array double[] mit den Quoten aller Beteiligten für eine
	 *            Zeile
	 * @return true, wenn der BaumbachBeteiligte unter index in einer
	 *         Gesamtschuldnerschaft ist, die eine Kostenquote > 0 tragen muss
	 */
	private boolean hatDarueberHinaus(int index, double[] spalten) {
		// boolean zwischen = false;
		// ArrayList<Integer> liste = baumbachBeteiligtenListe
		// .findeDarueberHinaus(index);
		// if (liste != null && liste.size() > 0 && spalten != null)
		// for (int i : liste) {
		// if (spalten[i] > 0.0)
		// zwischen = true;
		// }
		// return zwischen;
		final ArrayList<Integer> darueberHinaus = findeDarueberHinaus(index,
				spalten);
		return darueberHinaus != null && darueberHinaus.size() > 0
				&& spalten != null;
	}

	/**
	 * Findet über die BaumbachBeteiligtenListe alle Gesamtschuldnerschaften, in
	 * denen der BaumbachBeteiligte mit dem Index beteiligter ist, dann gucken,
	 * ob sie > 0% tragen müssen
	 * 
	 * @param index
	 *            die Position in der BaumbachBeteiligtenListe
	 * @param spalten
	 *            ein Array double[] mit den Quoten aller Beteiligten für eine
	 *            Zeile
	 * @return eine ArrayList mit den Indexen von allen Gesamtschuldnerschaften,
	 *         die eine Kostenquote > 0 tragen müssen, in denen der
	 *         BaumbachBeteiligte unter index in einer Gesamtschuldnerschaft
	 *         verurteilt ist
	 */
	private ArrayList<Integer> findeDarueberHinaus(int index, double[] spalten) {
		ArrayList<Integer> liste = baumbachBeteiligtenListe
				.findeDarueberHinaus(index);
		ArrayList<Integer> darueberHinaus = new ArrayList<Integer>();
		if (liste != null && liste.size() > 0 && spalten != null)
			for (int i : liste) {
				if (spalten[i] > 0.0)
					darueberHinaus.add(new Integer(i));
			}
		return darueberHinaus;
	}

	/**
	 * Die Methode dient dazu, die gesamte mögliche Quote zurückzugeben. Es wird
	 * die zusätzliche Quote zurückgegeben, die sich aus den
	 * BaumbachGesamtschuldnern ergibt, die auch den referenzierten
	 * BaumbachBeteiligten repräsentieren.
	 * 
	 * {@link BaumbachBeteiligtenListe#getUnterliegenFor(int, boolean, boolean)}
	 * 
	 * @param index
	 *            die Position in der BaumbachBeteiligtenListe
	 * @param spalten
	 *            ein Array double[] mit den Quoten aller Beteiligten für eine
	 *            Zeile
	 * @return die Summe als double
	 */
	private double berechneGanzeQuote(int index, double[] spalten) {
		/* Erst einmal trägt er, was er selbst trägt */
		double ganzeQuote = spalten[index];
		final ArrayList<Integer> darueberHinaus = findeDarueberHinaus(index,
				spalten);
		/* Und dann, wozu er gesamtschuldnerisch mit anderen verurteilt wurde */
		if (darueberHinaus != null && darueberHinaus.size() > 0) {
			for (Integer i : darueberHinaus)
				ganzeQuote += spalten[i];
		}
		return ganzeQuote;
	}

	/**
	 * Gibt den ganzen Kostentenor.
	 * 
	 * @return den Kostentenor als String
	 * 
	 * @see eu.gronos.kostenrechner.UrteilsTenorierend#erzeugeKostentenor()
	 */
	@Override
	public String erzeugeKostentenor() {
		String kostenTenor = "";
		// final double[][] quoten = ermittleQuoten();
		final double[][] quoten = getKosten();
		String[] kostenTenorSaetze = new String[quoten.length];
		/*
		 * Dann gucken, ob irgendwas genauso verteilt wird wie die
		 * Gerichtskosten.
		 */
		ArrayList<Integer> gleichlautendezeilen = findeGleichlautendeQuoten(quoten);
		for (int zeile = 0; zeile < quoten.length; zeile++) {
			/*
			 * Einleitung basteln: Die erste Zeile (0) ist für die
			 * Gerichtskosten, die weiteren je für einen Beteiligten, außer für
			 * BaumbachGesamtschuldner, die bekommen einen null-String
			 */
			if (zeile == 0) {
				if (gleichlautendezeilen == null
						|| gleichlautendezeilen.size() < 1)
					kostenTenorSaetze[zeile] = UrteilsTenorierend.DIE_GERICHTSKOSTEN_HABEN;
				else if (gleichlautendezeilen.size() == quoten.length - 1)
					/* Wenn alle Kosten gleich verteilt sind: */
					kostenTenorSaetze[zeile] = UrteilsTenorierend.DIE_KOSTEN_DES_RECHTSSTREITS_HABEN;
				else {
					/*
					 * wenn etwas genauso verteilt ist wie die Gerichtskosten:
					 * anhängen
					 */
					kostenTenorSaetze[zeile] = UrteilsTenorierend.DIE_GERICHTSKOSTEN;
					for (Integer i : gleichlautendezeilen)
						kostenTenorSaetze[zeile] += (UrteilsTenorierend.AUSSERGERICHTLICHE_KOSTEN_TEIL + baumbachBeteiligtenListe
								.parteiBezeichner(Beteiligter.GENITIV,
										i.intValue() - 1));
					/*
					 * TODO komisch, wenn einer allein die Kosten trägt:
					 * mehrfaches "und" - aber besser als nur Kommata
					 */
					kostenTenorSaetze[zeile] = TenorUtils
							.undStattKomma(kostenTenorSaetze[zeile])
							+ " haben ";
				}
			} else if (!baumbachBeteiligtenListe
					.isGesamtschuldnerschaft(zeile - 1)
					/*
					 * zu den Kosten, die wie die Gerichtskosten verteilt
					 * werden, keine eigene Zeile ausgeben.
					 */
					&& (gleichlautendezeilen == null || !gleichlautendezeilen
							.contains(zeile)))
				kostenTenorSaetze[zeile] = String.format(
						UrteilsTenorierend.DIE_AUSSERG_KOSTEN,
						baumbachBeteiligtenListe.parteiBezeichner(
								Beteiligter.GENITIV, zeile - 1));
			else
				kostenTenorSaetze[zeile] = null;
			// Die Spalten mit den BaumbachGesamtschuldner müssen an den
			// Anfang der Liste ihres Beteiligtentyps
			for (int spalte : baumbachBeteiligtenListe
					.beteiligteInTenorReihenfolge(true))
				if (kostenTenorSaetze[zeile] != null
						&& quoten[zeile][spalte] > 0.0
						&& quoten[zeile][spalte] < 100.0) {
					// "darüber hinaus"
					String darueberHinaus = hatDarueberHinaus(spalte,
							quoten[zeile]) ? UrteilsTenorierend.DARUEBER_HINAUS
							: "";
					kostenTenorSaetze[zeile] += String.format(
							UrteilsTenorierend.PARTEI_ZU_PROZENT,
							baumbachBeteiligtenListe.parteiBezeichner(
									Beteiligter.NOMINATIV, spalte)
									+ darueberHinaus, quoten[zeile][spalte]);
				} else if (kostenTenorSaetze[zeile] != null
						&& quoten[zeile][spalte] == 100.0) {
					/*
					 * Wenn eine Partei 100% trägt, nur den Bezeichner ausgeben.
					 */
					kostenTenorSaetze[zeile] += baumbachBeteiligtenListe
							.parteiBezeichner(Beteiligter.NOMINATIV, spalte);
				}
			// Gesamt-Kostentenor ergänzen
			if (kostenTenorSaetze[zeile] != null)
				kostenTenor += TenorUtils.hatStattHaben(TenorUtils
						.undStattKomma(TenorUtils
								.letztesKommaWeg(kostenTenorSaetze[zeile])))
						+ UrteilsTenorierend.ZU_TRAGEN;
		}
		if (!kostenTenor.contains(DIE_KOSTEN_DES_RECHTSSTREITS_HABEN)
				&& !kostenTenor.contains(TenorUtils
						.hatStattHaben(DIE_KOSTEN_DES_RECHTSSTREITS_HABEN)))
			kostenTenor += UrteilsTenorierend.KOSTENTRAGUNG_SELBST;
		return kostenTenor;
	}

	/**
	 * Der Getter für {@link #kosten} gibt den Inhalt selbigen Felds zurück. Ist
	 * dieses noch <code>null</code>, so wird vorher noch
	 * {@link #ermittleQuoten()} aufgerufen, um es zu füllen und
	 * {@link #entferneUnwesentlichesUnterliegen(double[])
	 * entferneUnwesentlichesUnterliegen}, um ggfls. ein unwesentliches
	 * Unterliegen zu entfernen.
	 * 
	 * @return gibt {@link #kosten} als double[][] zurück
	 */
	double[][] getKosten() {
		if (kosten == null) {
			kosten = ermittleQuoten();
			if (b92ii)
				for (int zeile = 0; zeile < kosten.length; zeile++) {
					/*
					 * Erst einmal die Quoten untersuchen und unwesentliches
					 * Unterliegen anderweitig verteilen (wenn ausgewählt)
					 */
					kosten[zeile] = entferneUnwesentlichesUnterliegen(kosten[zeile]);
				}
		}
		// FIXME nur Debug
		for (double d[] : kosten) {
			double summe = 0.0;
			for (double dd : d) {
				System.out.printf("%,.2f; ", dd);
				summe += dd;
			}
			System.out.printf("(Summe: %,.2f).%n", summe);
		}
		// FIXME Ende Debug
		return kosten;
	}

	/**
	 * Die Methode erzeugeTenor dient dazu, einen Tenor zu erzeugen, der auch
	 * nur aus dem Kostentenor bestehen kann. Hier besteht er mal aus dem
	 * Kostentenor und einer Streitwertfestsetzung.
	 * 
	 * @return einen String mit Hauptsachetenor, Kostentenor und
	 *         Streitwert-Festsetzung
	 * 
	 * @see eu.gronos.kostenrechner.UrteilsTenorierend#erzeugeTenor()
	 */
	@Override
	public String erzeugeTenor() {
		return erzeugeHauptsachetenor() + erzeugeKostentenor()
				+ erzeugeVollstreckbarkeitsTenor()
				+ erzeugeStreitwertFestsetzung();
	}

	/**
	 * Die Methode erzeugt eine Streitwertfestsetzung, indem der Streitwert auf
	 * den höchsten Einzelstreitwert der baumbachBeteiligtenListe festgesetzt
	 * wird.
	 * 
	 * @return einen formatierten String nach dem Muster
	 *         "Der Streitwert wird auf %,.2f EUR festgesetzt.%n"
	 * 
	 * @see eu.gronos.kostenrechner.UrteilsTenorierend#erzeugeStreitwertFestsetzung()
	 */
	@Override
	public String erzeugeStreitwertFestsetzung() {
		if (baumbachBeteiligtenListe.enthaeltAnWiderklageBeteiligte()) {
			// final double klSw = baumbachBeteiligtenListe
			// .getHoechstenStreitwert(Beteiligter.BEKLAGTE);
			// final double wkSw = baumbachBeteiligtenListe
			// .getHoechstenStreitwert(Beteiligter.KLAEGER) >
			// baumbachBeteiligtenListe
			// .getHoechstenStreitwert(Beteiligter.DRITTWIDERBEKLAGTE) ?
			// baumbachBeteiligtenListe
			// .getHoechstenStreitwert(Beteiligter.KLAEGER)
			// : baumbachBeteiligtenListe
			// .getHoechstenStreitwert(Beteiligter.DRITTWIDERBEKLAGTE);
			// final double sw = !wirtschaftlichIdentisch ? klSw + wkSw
			// : klSw > wkSw ? klSw : wkSw;
			double[] streitwerte = errechneStreitwert(true);
			/* Bei einer Widerklage müssen beide Streitwerte ausgegeben werden */
			return String.format(
					UrteilsTenorierend.STREITWERT_KLAGE_WIDERKLAGE,
					streitwerte[0], streitwerte[1], streitwerte[2]);
		} else
			// Ohne Widerklage muss nur ein Streitwert ausgegeben werden
			return String.format(UrteilsTenorierend.STREITWERT_KLAGE,
					baumbachBeteiligtenListe
							.getHoechstenStreitwert(Beteiligter.BEKLAGTE));
	}

	/**
	 * Die Methode dient dazu, den Streitwert zu ermitteln, indem der Streitwert
	 * auf den höchsten Einzelstreitwert der baumbachBeteiligtenListe
	 * festgesetzt wird.
	 * 
	 * @param mitWiderklage
	 *            wenn eine
	 *            {@link BaumbachBeteiligtenListe#enthaeltAnWiderklageBeteiligte()
	 *            Widerklage vorhanden} ist
	 * @return wenn mitWiderklage, dann ein Array double[3], bei dem 0 der
	 *         Streitwert insgesamt, 1 der Klagestreitwert und 2 der
	 *         Widerklagestreitwert sind; sonst double[1] nur mit dem
	 *         Streitwert.
	 */
	private double[] errechneStreitwert(boolean mitWiderklage) {
		// Bei einer Widerklage müssen beide Streitwerte ermittelt werden
		if (mitWiderklage) {
			final double klSw = baumbachBeteiligtenListe
					.getHoechstenStreitwert(Beteiligter.BEKLAGTE);
			final double wkSw = baumbachBeteiligtenListe
					.getHoechstenStreitwert(Beteiligter.KLAEGER) > baumbachBeteiligtenListe
					.getHoechstenStreitwert(Beteiligter.DRITTWIDERBEKLAGTE) ? baumbachBeteiligtenListe
					.getHoechstenStreitwert(Beteiligter.KLAEGER)
					: baumbachBeteiligtenListe
							.getHoechstenStreitwert(Beteiligter.DRITTWIDERBEKLAGTE);
			final double sw = !wirtschaftlichIdentisch ? klSw + wkSw
					: klSw > wkSw ? klSw : wkSw;
			return new double[] { sw, klSw, wkSw };
		} else {
			/* Ohne Widerklage muss nur ein Streitwert ermittelt werden */
			return new double[] { baumbachBeteiligtenListe
					.getHoechstenStreitwert(Beteiligter.BEKLAGTE) };
		}
	}

	/**
	 * @see eu.gronos.kostenrechner.UrteilsTenorierend#getTenorBeschreibung()
	 */
	@Override
	public String getTenorBeschreibung() {
		return BESCHREIBUNG;
	}

	/**
	 * Findet alle Kostenverteilungen von außergerichtlichen Kosten, die mit der
	 * Verteilung der Gerichtskosten völlig identisch sind (z.B. werden ohne
	 * Widerklage die Gerichtskosten und die außergerichtlichen Kosten des
	 * Klägers oft gleich verteilt oder eine Partei muss z.B. alles tragen).
	 * 
	 * @param quoten
	 *            ein Array double[][] mit allen Kostenquoten
	 * @return eine ArrayList<Integer> mit allen Zeilen, die identisch sind
	 *         (dürfte meist aber nur eine Zeile sein).
	 */
	private ArrayList<Integer> findeGleichlautendeQuoten(double[][] quoten) {
		ArrayList<Integer> liste = new ArrayList<Integer>();
		if (quoten == null || quoten.length < 1)
			return null;
		for (int zeile = 0; zeile < quoten.length; zeile++) {
			if (zeile > 0 && istGleichlautendeQuote(quoten[0], quoten[zeile]))
				liste.add(new Integer(zeile));
		}
		return liste;
	}

	/**
	 * Prüft, ob die beiden double[]-Arrays identisch sind, um zu gucken, ob die
	 * außergerichtlichen Kosten eines bestimmten Beteiligten genauso verteilt
	 * sind wie die Gerichtskosten.
	 * 
	 * @param gerichtskostenQuoten
	 *            double[]-Array mit der Verteilung der Gerichtskosten
	 * @param andereQuoten
	 *            double[]-Array mit der Verteilung der außergerichtlichen
	 *            Kosten eines Beteiligten
	 * @return true, wenn identisch
	 */
	private boolean istGleichlautendeQuote(double[] gerichtskostenQuoten,
			double[] andereQuoten) {
		boolean zwischen = true;
		for (int spalte = 0; spalte < andereQuoten.length; spalte++) {
			/*
			 * Muss auf zwei Nachkommastellen runden, sonst erkennt er Identität
			 * nicht immer wegen Double-Rundungsfehlern.
			 */
			if (Math.round(gerichtskostenQuoten[spalte] * 100) != Math
					.round(andereQuoten[spalte] * 100))
				zwischen = false;
		}
		return zwischen;
	}

	/**
	 * Die Methode gibt die Begründung für den Kostentenor als String zurück.
	 * 
	 * @return den Text Gründe/Entscheidungsgründe als String
	 * 
	 * @see eu.gronos.kostenrechner.UrteilsTenorierend#getGruende()
	 */
	@Override
	public String getGruende() {
		return gruende.toString();
	}

	/**
	 * Die Methode setzt die Entscheidungsgründe zurück (sofern nicht leer) und
	 * beginnt sie neu mit dem übergebenen Text.
	 * 
	 * @param text
	 *            eine {@link CharSequence} mit dem Anfangssatz(-bestandteil)
	 *            der Gründe.
	 */
	private void starteGruende(CharSequence text) {
		if (gruende.length() > 0)
			gruende.delete(0, gruende.length());
		erweitereGruende(text);
	}

	/**
	 * Die Methode erweitert die bisherigen Gründe
	 * 
	 * @param text
	 *            eine {@link CharSequence} mit den weiteren Gründen.
	 */
	private void erweitereGruende(CharSequence text) {
		gruende.append(text);
	}

	/**
	 * Die Methode dient dazu, über {@link #baueTextAusTabelle()} die Gründe für
	 * die BaumbachFormel zu basteln.
	 */
	private void baueGruende() {
		starteGruende(GRUENDE_91_92_100II);
		baueTextAusTabelle();
	}

	/**
	 * Die Methode dient dazu, aus der {@link #zeilenListe} einen Text mit
	 * Tabulatoren zu basteln, der von der {@link JTable} im {@link TenorDialog}
	 * überlagert werden kann. Gleichzeitig wird auch
	 * {@link eu.gronos.kostenrechner.Tabulierend#getRange() range} gesetzt.
	 */
	private void baueTextAusTabelle() {
		range[0] = getGruende().length() - 1;
		for (Object[] zeile : zeilenListe) {
			erweitereGruende(TenorUtils.zeileZuText(zeile));
		}
		range[1] = getGruende().length() - 1;
	}

	/**
	 * Die Methode dient dazu, eine "Tabelle" (also eine ArrayList aus Arrays
	 * Object[]) zu erzeugen, das dann zur Begründung anhand der Einzelangriffe
	 * genutzt werden kann.
	 * 
	 * @param quoten
	 *            die Gerichtskostenquoten als double[][]
	 * @param fiktiverStreitwert
	 *            ein double mit dem fiktiven Streitwert der Gerichtskosten
	 * @return eine ArrayList aus Arrays Object[], das jeweils zum Teil mit
	 *         String-, zum Teil mit Double gefüllt ist.
	 */
	private ArrayList<Object[]> baueAngriffsTabelle(double[] quoten,
			double fiktiverStreitwert) {
		/*
		 * Durch die Verwendung einer ArrayList muss die Anzahl der Zeilen nicht
		 * ermittelt werden, aber eine Liste der Angriffe muss über
		 * beteiligteWieVerklagt ermittelt werden.
		 */
		zeilenListe = new ArrayList<Object[]>();
		final ArrayList<Integer> beteiligtenIndicesZeilen = baumbachBeteiligtenListe
				.beteiligteWieVerklagt();
		/*
		 * Liste der Beteiligten/Gesamtschuldnerschaften über
		 * beteiligteInTenorReihenfolge für Zahl der Spalten ermitteln
		 */
		final ArrayList<Integer> beteiligtenIndicesSpalten = baumbachBeteiligtenListe
				.beteiligteInTenorReihenfolge(true);
		/* Hierin werden die Verluste der Beteiligten zum Summieren gespeichert */
		final double[] verluste = new double[beteiligtenIndicesSpalten.size()];
		for (int i = 0; i < verluste.length; i++)
			verluste[i] = 0.0;
		/* Kopfzeile und Ergebniszeile bauen und der ArrayList hinzufügen */
		final Object[] kopfObjekt = new Object[beteiligtenIndicesSpalten.size() + 1];
		kopfObjekt[0] = KOPFZEILE_BEGINN;
		/* Kopfzeile mithilfe einer For-Schleife füllen */
		for (int spalte = 0; spalte < beteiligtenIndicesSpalten.size(); spalte++) {
			kopfObjekt[spalte + 1] = baumbachBeteiligtenListe.parteiBezeichner(
					Beteiligter.GENITIV, beteiligtenIndicesSpalten.get(spalte)
							.intValue());
		}
		zeilenListe.add(kopfObjekt);
		/*
		 * ArrayList füllen: äußere Schleife: für jede Zeile ein Array Object[]
		 * anlegen mit 1*String und 1..n*Double
		 */
		for (int zeile = 0; zeile < beteiligtenIndicesZeilen.size(); zeile++) {
			/*
			 * Das Array Object[] mit den Spalten für die aktuelle Zeile bekommt
			 * eine Spalte mehr, als beteiligteInTenorReihenfolge zurück gibt,
			 * denn die 0. Spalte enthält eine Spaltenüberschrift
			 */
			final Object[] objects = new Object[beteiligtenIndicesSpalten
					.size() + 1];
			final int a = beteiligtenIndicesZeilen.get(zeile).intValue();
			final BaumbachBeteiligter bbtA = baumbachBeteiligtenListe.get(a);
			/* Die "Kopfspalte" links sagt Angriff wer gegen wen */
			objects[0] = baumbachBeteiligtenListe.benenneAngriffGegen(a);
			for (int spalte = 0; spalte < beteiligtenIndicesSpalten.size(); spalte++) {
				final int b = beteiligtenIndicesSpalten.get(spalte).intValue();
				final BaumbachBeteiligter bbtB = baumbachBeteiligtenListe
						.get(b);
				double wert = 0.0;
				/*
				 * Wert für innere Tabelle nach BeteiligtenTyp von a und b
				 * ermitteln
				 */
				if (a == b) {
					/*
					 * Wenn a und b identisch (dasselbe Element in der bbtl),
					 * dann bezieht sich das Unterliegen von b auf den Angriff
					 * gegen b, ergo: das Unterliegen von b ist hier richtig.
					 */
					wert = bbtB.getUnterliegen();
				} else if (bbtB.isGesamtschuldnerschaft()
						&& baumbachBeteiligtenListe
								.getAlsGesamtschuldner(b)
								.contains(
										bbtA.getTyp(),
										baumbachBeteiligtenListe
												.ermittleLaufendeNummerFuerBeteiligtenTyp(a))) {
					/*
					 * Wenn b eine gesamtschuldnerische Verurteilung ist und a
					 * (gegen den sich der Angriff richtet) darin enthalten ist,
					 * dann muss das Unterliegen der Gesamtschuldnerschaft in
					 * die Spalte.
					 */
					wert = bbtB.getUnterliegen();
				} else if (bbtA.getTyp() == Beteiligter.BEKLAGTE) {
					/*
					 * Wenn a Beklagter und b Kläger ist, dann Unterliegen
					 * berechnen
					 */
					if (bbtB.getTyp() == Beteiligter.KLAEGER)
						wert = bbtA.getStreitwert()
								- baumbachBeteiligtenListe.getUnterliegenFor(a,
										false, true);
				} else if ((bbtA.getTyp() == Beteiligter.KLAEGER && bbtA
						.isAnWiderklageBeteiligt())
						|| bbtA.getTyp() == Beteiligter.DRITTWIDERBEKLAGTE) {
					/*
					 * Wenn a Widerbeklagter ist und b Widerkläger, dann
					 * Unterliegen berechnen.
					 */
					if (bbtB.getTyp() == Beteiligter.BEKLAGTE
							&& bbtB.isAnWiderklageBeteiligt())
						wert = bbtA.getStreitwert()
								- baumbachBeteiligtenListe.getUnterliegenFor(a,
										false, true);
				}
				/*
				 * Den ermittelten Wert in die Spalte schreiben (Achtung! Das
				 * Object[] hat eine Spalte mehr als der Spaltenzähler!) und auf
				 * die Verluste addieren.
				 */
				objects[spalte + 1] = new Double(wert);
				verluste[spalte] += wert;
			}
			zeilenListe.add(objects);
		}
		/* Zeilen für Verlustsummen und Ergebnis erzeugen */
		final Object[] ergebnisObjekt = new Object[beteiligtenIndicesSpalten
				.size() + 1];
		ergebnisObjekt[0] = String.format(ERGEBNIS_ZEILE, fiktiverStreitwert);
		final Object[] verlusteObjekt = new Object[beteiligtenIndicesSpalten
				.size() + 1];
		verlusteObjekt[0] = VERLUST_GESAMT;
		/*
		 * Verlustsummen-Zeile und Ergebniszeile mithilfe einer For-Schleife
		 * füllen
		 */
		for (int spalte = 0; spalte < beteiligtenIndicesSpalten.size(); spalte++) {
			verlusteObjekt[spalte + 1] = new Double(verluste[spalte]);
			ergebnisObjekt[spalte + 1] = new Double(
					quoten[beteiligtenIndicesSpalten.get(spalte).intValue()]);
		}
		/* Verlustsummen-Zeile und Ergebniszeile hinzufügen */
		zeilenListe.add(verlusteObjekt);
		zeilenListe.add(ergebnisObjekt);
		return zeilenListe;
	}

	@Override
	public Object[][] getTableCellValues() {
		return TenorUtils.convertTableCellValues(zeilenListe);
	}

	@Override
	public Class<?>[] getColumnClasses() {
		Class<?>[] columnClasses;
		if (zeilenListe == null || zeilenListe.size() < 1
				|| zeilenListe.get(0).length < 1)
			return null;
		Object[] tabellenKopf = zeilenListe.get(0);
		columnClasses = new Class<?>[tabellenKopf.length];
		columnClasses[0] = String.class;
		for (int i = 1; i < tabellenKopf.length; i++)
			columnClasses[i] = Double.class;
		return columnClasses;
	}

	@Override
	public String[] getColumnNames() {
		return TenorUtils.convertColumnNames(zeilenListe);
	}

	@Override
	public int[] getRange() {
		return this.range;
	}

	/**
	 * Die Methode erzeugt den Tenor zur vorläufigen Vollstreckbarkeit
	 * 
	 * @return einen String
	 */
	@Override
	public String erzeugeVollstreckbarkeitsTenor() {
		String schuldner = UrteilsTenorierend.JEWEILIGER_VOLLSTRECKUNGSSCHULDNER;
		String glaeubiger = UrteilsTenorierend.JEWEILIGER_VOLLSTRECKUNGSGLAEUBIGER;
		if (vollstreckbarkeit == null)
			vollstreckbarkeit = berechneVollstreckbarkeit();
		erweitereGruende(vollstreckbarkeit.toString());
		if (vollstreckbarkeit.sindAlle708iVm713()) {
			return UrteilsTenorierend.URTEIL_VORLAEUFIG_VOLLSTRECKBAR;
		} else if (vollstreckbarkeit.sindAlle709()) {
			return UrteilsTenorierend.URTEIL_VORLAEUFIG_VOLLSTRECKBAR_709;
		} else if (vollstreckbarkeit.sindAlle708iVm711()) {
			ArrayList<Integer> werIst708iVm711 = vollstreckbarkeit
					.werIst708iVm711();
			/*
			 * Wenn es nur ein solches Verhältnis gibt, dann die Parteien
			 * richtig ausgeben.
			 */
			if (werIst708iVm711.size() == 1) {
				Beteiligter schuldnerBeteiligter = ((BaumbachVollstreckbarkeitsObjekt) vollstreckbarkeit
						.get(werIst708iVm711.get(0))).getSchuldner();
				schuldner = schuldnerBeteiligter.parteiBezeichner(
						Beteiligter.DATIV, schuldnerBeteiligter.getLfdNr(),
						false);
				Beteiligter glaeubigerBeteiligter = ((BaumbachVollstreckbarkeitsObjekt) vollstreckbarkeit
						.get(werIst708iVm711.get(0))).getGlaeubiger();
				glaeubiger = glaeubigerBeteiligter.parteiBezeichner(
						Beteiligter.DATIV, glaeubigerBeteiligter.getLfdNr(),
						false);
			}
			/* Sonst bleibt's beim "jeweiligen" Gläubiger/Schuldner */
			return UrteilsTenorierend.URTEIL_VORLAEUFIG_VOLLSTRECKBAR
					+ String.format(
							UrteilsTenorierend.URTEIL_VORLAEUFIG_VOLLSTRECKBAR_708_711,
							TenorUtils.satzBeginn(schuldner), glaeubiger);
		}
		StringBuffer tenor = new StringBuffer(
				UrteilsTenorierend.URTEIL_VORLAEUFIG_VOLLSTRECKBAR);
		/*
		 * Bei gemischtem Tenor erst die 709-er, dann die 708/711-er
		 * Verhältnisse ausgeben.
		 */
		ArrayList<Integer> werIst709 = vollstreckbarkeit.werIst709();
		if (werIst709.size() == 1) {
			Beteiligter schuldnerBeteiligter = ((BaumbachVollstreckbarkeitsObjekt) vollstreckbarkeit
					.get(werIst709.get(0))).getSchuldner();
			Beteiligter glaeubigerBeteiligter = ((BaumbachVollstreckbarkeitsObjekt) vollstreckbarkeit
					.get(werIst709.get(0))).getGlaeubiger();
			tenor.append(String.format(
					"Für %s gegenüber %s",
					glaeubigerBeteiligter.parteiBezeichner(
							Beteiligter.AKKUSATIV,
							glaeubigerBeteiligter.getLfdNr(),
							baumbachBeteiligtenListe
									.zaehleBeteiligteFuerTyp(glaeubigerBeteiligter
											.getTyp()) < 2),
					schuldnerBeteiligter
							.parteiBezeichner(
									Beteiligter.DATIV,
									schuldnerBeteiligter.getLfdNr(),
									baumbachBeteiligtenListe
											.zaehleBeteiligteFuerTyp(schuldnerBeteiligter
													.getTyp()) < 2))
					+ UrteilsTenorierend.URTEIL_VORLAEUFIG_VOLLSTRECKBAR_709_EINZELN);
		} else if (werIst709.size() > 0) {
			tenor.append(vollstreckbarkeit.benenneWerIst(werIst709,
					baumbachBeteiligtenListe)
					+ UrteilsTenorierend.URTEIL_VORLAEUFIG_VOLLSTRECKBAR_709_EINZELN);
		}
		ArrayList<Integer> werIst708iVm711 = vollstreckbarkeit
				.werIst708iVm711();
		if (werIst708iVm711.size() == 1) {
			Beteiligter schuldnerBeteiligter = ((BaumbachVollstreckbarkeitsObjekt) vollstreckbarkeit
					.get(werIst708iVm711.get(0))).getSchuldner();
			Beteiligter glaeubigerBeteiligter = ((BaumbachVollstreckbarkeitsObjekt) vollstreckbarkeit
					.get(werIst708iVm711.get(0))).getGlaeubiger();
			tenor.append(String
					.format(UrteilsTenorierend.URTEIL_VORLAEUFIG_VOLLSTRECKBAR_708_711,
							TenorUtils.satzBeginn(schuldnerBeteiligter
									.parteiBezeichner(
											Beteiligter.DATIV,
											schuldnerBeteiligter.getLfdNr(),
											baumbachBeteiligtenListe
													.zaehleBeteiligteFuerTyp(schuldnerBeteiligter
															.getTyp()) < 2)),
							glaeubigerBeteiligter
									.parteiBezeichner(
											Beteiligter.NOMINATIV,
											glaeubigerBeteiligter.getLfdNr(),
											baumbachBeteiligtenListe
													.zaehleBeteiligteFuerTyp(glaeubigerBeteiligter
															.getTyp()) < 2)));
		} else if (werIst708iVm711.size() > 0) {
			tenor.append(String
					.format(UrteilsTenorierend.URTEIL_VORLAEUFIG_VOLLSTRECKBAR_708_711_WERIST,
							vollstreckbarkeit.benenneWerIst(werIst708iVm711,
									baumbachBeteiligtenListe),
							UrteilsTenorierend.JEWEILIGER_VOLLSTRECKUNGSSCHULDNER,
							UrteilsTenorierend.JEWEILIGER_VOLLSTRECKUNGSGLAEUBIGER));
		}
		return tenor.toString();
	}

	/**
	 * Die Methode berechnet, inwieweit das Urteil vorläufig vollstreckbar ist.
	 * Vorläufige Vollstreckbarkeit: erst einmal gucken, ob die Hauptsache über
	 * die Schwelle von §709 kommt. Wenn nicht, dann bei den Kosten rechnen:
	 * Array double[][] durchgehen, Quote mit jeweiligen Kosten multiplizieren
	 * und für jede Partei zusammen rechnen. Die jeweils eigenen Kosten dabei
	 * auslassen.
	 * 
	 * @return eine {@link VollstreckbarkeitsListe}, deren 0. Element die
	 *         Unanfechtbarkeit speichert, die weiteren Elemente speichern die
	 *         vorläufige Vollstreckbarkeit für die
	 *         Einzelvollstreckungsverhältnisse.
	 */
	private VollstreckbarkeitsListe berechneVollstreckbarkeit() {
		VollstreckbarkeitsListe vollstreckbarkeit = new VollstreckbarkeitsListe();
		/* Unanfechtbarkeit prüfen und der Liste hinzufügen */
		vollstreckbarkeit.add(pruefeUnanfechtbarkeit());
		/* Nach festgestellter Unanfechtbarkeit war's das schon. */
		if (vollstreckbarkeit.sindAlle708iVm713())
			return vollstreckbarkeit;
		/*
		 * Einzelne Vollstreckungsverhältnisse durchgehen und
		 * BaumbachVollstreckbarkeitsObjekt der Liste hinzufügen; an quoten[][]
		 * entlanghangeln.
		 */
		double[][] quoten = getKosten();
		final long streitwert = (long) errechneStreitwert(baumbachBeteiligtenListe
				.enthaeltAnWiderklageBeteiligte())[0];
		for (int glaeubigerIndex = 0; glaeubigerIndex < baumbachBeteiligtenListe
				.size(); glaeubigerIndex++) {
			for (int schuldnerIndex = 0; schuldnerIndex < baumbachBeteiligtenListe
					.size(); schuldnerIndex++) {
				BaumbachBeteiligter glaeubiger = baumbachBeteiligtenListe
						.get(glaeubigerIndex);
				BaumbachBeteiligter schuldner = baumbachBeteiligtenListe
						.get(schuldnerIndex);
				if (glaeubigerIndex != schuldnerIndex
						&& !(glaeubiger instanceof BaumbachGesamtschuldnerschaft)
						&& !(schuldner instanceof BaumbachGesamtschuldnerschaft)) {
					double hauptsache = 0;
					double vollstreckbareKosten = 0;
					if (glaeubiger.getTyp() == Beteiligter.KLAEGER) {
						/* Bei einem Kläger vorher die Gerichtskosten drauf */
						vollstreckbareKosten += berechneGanzeQuote(
								schuldnerIndex, quoten[0])
								/ 100.0
								* StreitwertReduktion.errechneGebuehr(
										GerichtsGebuehrenTabelle.KV1210,
										streitwert);
					}
					/*
					 * Die Anwaltskosten (KOSTEN_BEKLAGTER) immer - auch bei Kl
					 * und Dwbk
					 */
					vollstreckbareKosten += berechneGanzeQuote(schuldnerIndex,
							quoten[glaeubigerIndex + 1])
							/ 100.0
							* (long) (TenorUtils.errechneGebuehrenSumme(
									UrteilsTenorierend.KOSTEN_BEKLAGTER,
									streitwert));
					// hauptsache berechnen, wenn es in der "Paarung" eine
					// Hauptsache gibt
					if ((Beteiligter.KLAEGER == glaeubiger.getTyp() && Beteiligter.BEKLAGTE == schuldner
							.getTyp())
							|| (Beteiligter.BEKLAGTE == glaeubiger.getTyp()
									&& glaeubiger.isAnWiderklageBeteiligt()
									&& schuldner.isAnWiderklageBeteiligt() && (Beteiligter.KLAEGER == schuldner
									.getTyp() || Beteiligter.DRITTWIDERBEKLAGTE == schuldner
									.getTyp()))) {
						hauptsache = baumbachBeteiligtenListe.getUnterliegenFor(
								schuldnerIndex, false, true);
					}
					vollstreckbarkeit.add(BaumbachVollstreckbarkeitsObjekt
							.pruefeSicherheitsleistung((long) hauptsache,
									(long) vollstreckbareKosten, glaeubiger,
									schuldner));
				}
			}
		}
		return vollstreckbarkeit;
	}

	/**
	 * Die Methode prüft, ob das Urteil unanfechtbar ist (§§ 708, 713 ZPO).
	 * 
	 * @return ein {@link VollstreckbarkeitsObjekt}, das im Fall der
	 *         Unanfechtbarkeit 708 und 713 auf <code>true</code> setzt.
	 *         Ansonsten enthaelt es nichts.
	 */
	private VollstreckbarkeitsObjekt pruefeUnanfechtbarkeit() {
		/*
		 * Wenn der Streitwert schon nicht über 600 EUR geht, ist das Urteil
		 * unanfechtbar.
		 */
		if (errechneStreitwert(baumbachBeteiligtenListe
				.enthaeltAnWiderklageBeteiligte())[0] <= UrteilsTenorierend.BERUFUNGS_GRENZE) {
			return new VollstreckbarkeitsObjekt(true, false, false, false, true);
		} else if (errechneStreitwert(baumbachBeteiligtenListe
				.enthaeltAnWiderklageBeteiligte())[0] <= 2 * UrteilsTenorierend.BERUFUNGS_GRENZE) {
			/*
			 * Beschwer jedes Einzelbeteiligten überprüfen, ob nicht über
			 * Berufungsgrenze
			 */
			for (int index = 0; index < baumbachBeteiligtenListe.size(); index++)
				if (!baumbachBeteiligtenListe.isGesamtschuldnerschaft(index)) {
					/* erstmal das "eigene" "ganze" Unterliegen */
					double unterliegen = baumbachBeteiligtenListe
							.getUnterliegenFor(index, false, true);
					/*
					 * Wenn der beteiligte ein "Angreifer" ist:
					 */
					/* Unterschleife beim Kläger ggü Bk. */
					if (baumbachBeteiligtenListe.getTypFor(index) == Beteiligter.KLAEGER) {
						double angreiferUnterliegen = 0.0;
						for (int angriff = 0; angriff < baumbachBeteiligtenListe
								.size(); angriff++) {
							if (baumbachBeteiligtenListe.getTypFor(angriff) == Beteiligter.BEKLAGTE
									&& !baumbachBeteiligtenListe
											.isGesamtschuldnerschaft(angriff)) {
								final double angriffsUnterliegen = baumbachBeteiligtenListe
										.getStreitwertFor(angriff)
										- baumbachBeteiligtenListe
												.getUnterliegenFor(angriff, false,
														true);
								/*
								 * Nach Vorhandensein von
								 * Gesamtschuldnerschaften unterscheiden
								 */
								if (baumbachBeteiligtenListe
										.enthaeltGesamtschuldner()
										&& angreiferUnterliegen < angriffsUnterliegen) {
									angreiferUnterliegen = angriffsUnterliegen;
								} else if (baumbachBeteiligtenListe
										.enthaeltGesamtschuldner()) {
									// nüscht
								} else {
									angreiferUnterliegen += angriffsUnterliegen;
								}
							}
						}
						unterliegen += angreiferUnterliegen;
					}
					/* Unterschleife beim Bk/Widerkläger ggü. Widerbeklagten */
					if (baumbachBeteiligtenListe.getTypFor(index) == Beteiligter.BEKLAGTE
							&& baumbachBeteiligtenListe
									.isAnWiderklageBeteiligt(index)) {
						double angreiferUnterliegen = 0.0;
						for (int angriff = 0; angriff < baumbachBeteiligtenListe
								.size(); angriff++) {
							if (!baumbachBeteiligtenListe
									.isGesamtschuldnerschaft(angriff)
									&& baumbachBeteiligtenListe
											.isAnWiderklageBeteiligt(angriff)
									&& (baumbachBeteiligtenListe
											.getTypFor(angriff) == Beteiligter.DRITTWIDERBEKLAGTE || baumbachBeteiligtenListe
											.getTypFor(angriff) == Beteiligter.KLAEGER)) {
								final double angriffsUnterliegen = baumbachBeteiligtenListe
										.getStreitwertFor(angriff)
										- baumbachBeteiligtenListe
												.getUnterliegenFor(angriff, false,
														true);
								/*
								 * Nach Vorhandensein von
								 * Gesamtschuldnerschaften unterscheiden
								 */
								if (baumbachBeteiligtenListe
										.enthaeltGesamtschuldner()
										&& angreiferUnterliegen < angriffsUnterliegen) {
									angreiferUnterliegen = angriffsUnterliegen;
								} else if (baumbachBeteiligtenListe
										.enthaeltGesamtschuldner()) {
									// nüscht
								} else {
									angreiferUnterliegen += angriffsUnterliegen;
								}
							}
						}
						unterliegen += angreiferUnterliegen;
					}
					/*
					 * Sobald irgend etwas höher als die Berufungsgrenze ist,
					 * ist das Urteil anfechtbar.
					 */
					if (unterliegen > UrteilsTenorierend.BERUFUNGS_GRENZE)
						return new VollstreckbarkeitsObjekt();
				}
			/*
			 * Wenn wir hier angekommen sind (also aus der Schleife vorher nicht
			 * ausgestiegen sind), ist das Urteil unanfechtbar
			 */
			return new VollstreckbarkeitsObjekt(true, false, false, false, true);
		} else {
			return new VollstreckbarkeitsObjekt();
		}
	}
}
