/**
 * KostenTenorLadenSpeichern.java
 * eu.gronos.kostenrechner (Kostenrechner)
 */
package eu.gronos.kostenrechner;

import java.awt.Component;
import java.io.File;
import java.util.ArrayList;
import java.util.GregorianCalendar;

import javax.swing.JFileChooser;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.xml.bind.JAXB;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;

import eu.gronos.kostenrechner.VerfahrensDatenContainer.Allgemein;
import eu.gronos.kostenrechner.VerfahrensDatenContainer.Streitgenossen;
import eu.gronos.kostenrechner.VerfahrensDatenContainer.Teilklageruecknahme;

/**
 * Eine Klasse, die alle Zustände des Programms im XML-Format speichern kann.
 *
 * @author Peter Schuster (setrok)
 * @date 26.12.2014
 *
 * @url "http://java.kompf.de/jaxbjava.html"
 * @url "http://docs.oracle.com/javase/tutorial/jaxb/intro/"
 */
class BerechnungLadenSpeichernDialog extends JFileChooser {

	private static final String FEHLER_BEIM_LADEN_DER_XML_DATEI = "Fehler beim Laden der XML-Datei.";
	private static final long serialVersionUID = 5816976530887334541L;
	static final String DATEITYP = "Schusters kleiner Kostentenor XML-Dateien";
	static final String DATEIENDUNG = "skktx";
	static final String DATEIENDUNG_GANZ = "." + DATEIENDUNG;
	static final FileNameExtensionFilter filter = new FileNameExtensionFilter(
			DATEITYP, DATEIENDUNG);

	/**
	 * Konstruktor ohne Parameter
	 * 
	 */
	BerechnungLadenSpeichernDialog() {
		super();
		setFileFilter(filter);
	}

	/**
	 * Die Methode dient zum Umwandeln der Java Objekte (dargereicht als
	 * {@link VerfahrensDatenContainer}) in XML
	 * 
	 * @param container
	 *            der {@link VerfahrensDatenContainer} mit allen zu speichernden
	 *            Zuständen
	 * @param datei
	 *            ein {@link File}, in das zu speichern ist
	 * @throws JAXBException
	 */
	private void speicherXml(VerfahrensDatenContainer container, File datei)
			throws JAXBException {
		JAXBContext context = JAXBContext
				.newInstance(VerfahrensDatenContainer.class);
		Marshaller marshaller = context.createMarshaller();
		marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
		marshaller.marshal(container, datei);
	}

	/**
	 * Die Methode dient dazu, einen Dateiauswahldialog der {@link JFileChooser
	 * Oberklasse} aufzurufen um dann mit
	 * {@link #speicherXml(VerfahrensDatenContainer, File) speicherXml} die
	 * übergebenen Programmzustände als XML zu speichern.
	 * 
	 * @param parent
	 *            die aufrufende {@link Component}
	 * @param container
	 *            der {@link VerfahrensDatenContainer} mit allen zu speichernden
	 *            Zuständen
	 * @return den Rückgabestatus des {@link JFileChooser}s:<br>
	 *         <ul>
	 *         <li>{@link JFileChooser#CANCEL_OPTION}</li>
	 *         <li>{@link JFileChooser#APPROVE_OPTION}</li>
	 *         <li>{@link JFileChooser#ERROR_OPTION}, wenn der Dialog einen
	 *         Fehler auslöst, er abgebrochen wird oder es einen Fehler beim
	 *         Laden gibt.</li>
	 * 
	 * @see JFileChooser#showSaveDialog(Component)
	 * @see JFileChooser#showDialog(Component, String)
	 */
	public int speicherXmlDialog(Component parent,
			VerfahrensDatenContainer container) {
		int option = super.showSaveDialog(parent);
		if (option == JFileChooser.APPROVE_OPTION)
			try {
				File datei = super.getSelectedFile();
				if (!datei.getName().endsWith(DATEIENDUNG_GANZ))
					datei = new File(datei.getAbsolutePath() + DATEIENDUNG_GANZ);
				container.allgemein.speicherdatum = speicherDatum();
				speicherXml(container, datei);
				System.out.println("Gespeichert: " + datei.getAbsolutePath());
			} catch (JAXBException je) {
				option = JFileChooser.ERROR_OPTION;
				Kostenrechner.zeigeFehler(FehlerDialog.FEHLER_BLANKO,
						"Fehler beim Speichern als XML!", je);
			}
		return option;
	}

	/**
	 * Die Methode dient dazu, das aktuelle Systemdatum zum Speichern zu
	 * ermitteln
	 * 
	 * @return einen {@link XMLGregorianCalendar} mit dem aktuellen Systemdatum
	 * @throws DatatypeConfigurationException
	 */
	private static XMLGregorianCalendar speicherDatum() {
		try {
			DatatypeFactory factory = DatatypeFactory.newInstance();
			return factory.newXMLGregorianCalendar(new GregorianCalendar());
		} catch (DatatypeConfigurationException dce) {
			Kostenrechner.zeigeFehler(FehlerDialog.FEHLER_BLANKO,
					"Schwerer Fehler beim Konfigurieren der Zeit!", dce);
			return null;
		}
	}

	/**
	 * Die Methode dient zum Umwandeln XML in einen
	 * {@link VerfahrensDatenContainer}
	 * 
	 * @param datei
	 *            ein {@link File}, das zu laden ist
	 * @return ein {@link VerfahrensDatenContainer} mit allen gespeicherten
	 *         Zuständen
	 * @throws JAXBException
	 */
	private VerfahrensDatenContainer unmarshalXml(File datei)
			throws JAXBException {
		return JAXB.unmarshal(datei, VerfahrensDatenContainer.class);
	}

	/**
	 * Die Methode, die den Container ausliest, muss bei
	 * {@link BaumbachGesamtschuldnerschaft}en die richtige
	 * {@link BaumbachBeteiligtenListe} hinzufügen
	 * 
	 * @param streitgenossen
	 *            ein Untercontainer
	 *            {@link VerfahrensDatenContainer.Streitgenossen}
	 * @return den reparierten Untercontainer
	 *         {@link VerfahrensDatenContainer.Streitgenossen}
	 */
	private VerfahrensDatenContainer.Streitgenossen fixGesamtschuldnerschaften(
			VerfahrensDatenContainer.Streitgenossen streitgenossen) {
		BaumbachBeteiligtenListe beklagte = streitgenossen.beklagte;
		if (beklagte.enthaeltGesamtschuldner())
			for (int index = 0; index < beklagte.size(); index++) {
				if (beklagte.get(index) instanceof BaumbachGesamtschuldnerschaft)
					((BaumbachGesamtschuldnerschaft) beklagte.get(index))
							.setBaumbachBeteiligtenListe(beklagte);
			}
		BaumbachBeteiligtenListe widerbeklagte = new BaumbachBeteiligtenListe();
		widerbeklagte.addAll(streitgenossen.klaeger);
		widerbeklagte.addAll(streitgenossen.drittwiderbeklagte);
		if (widerbeklagte.enthaeltGesamtschuldner())
			for (int index = 0; index < widerbeklagte.size(); index++) {
				if (widerbeklagte.get(index) instanceof BaumbachGesamtschuldnerschaft)
					((BaumbachGesamtschuldnerschaft) widerbeklagte.get(index))
							.setBaumbachBeteiligtenListe(widerbeklagte);
			}
		return streitgenossen;
	}

	/**
	 * Die Methode dient dazu, einen Dateiauswahldialog der {@link JFileChooser
	 * Oberklasse} aufzurufen um dann mit {@link #unmarshalXml(File)} aus der
	 * ausgewählte Datei einen {@link VerfahrensDatenContainer} auszulesen.
	 * 
	 * @param parent
	 *            die aufrufende {@link Component}
	 * @return einen {@link VerfahrensDatenContainer} zum Auslesen
	 * 
	 * @see JFileChooser#showOpenDialog(Component)
	 * @see JFileChooser#showDialog(Component, String)
	 * @see #unmarshalXml(File)
	 */
	public VerfahrensDatenContainer ladeXmlDialog(Component parent) {
		VerfahrensDatenContainer container = null;
		int option = super.showOpenDialog(parent);
		if (option == JFileChooser.APPROVE_OPTION) {
			File datei = super.getSelectedFile();
			container = ladeXml(datei);
		}
		return container;
	}

	/**
	 * Die Methode dient dazu, mit {@link #unmarshalXml(File)} aus der
	 * ausgewählten Datei einen {@link VerfahrensDatenContainer} auszulesen.
	 * 
	 * @param ein
	 *            {@link File}, das zu laden ist
	 * @return ein {@link VerfahrensDatenContainer} mit allen gespeicherten
	 *         Zuständen
	 * @see #unmarshalXml(File)
	 */
	public VerfahrensDatenContainer ladeXml(File datei) {
		VerfahrensDatenContainer container = null;
		try {
			container = unmarshalXml(datei);
			System.out.println("Geladen: " + datei.getAbsolutePath());
		} catch (JAXBException je) {
			Kostenrechner.zeigeFehler(FehlerDialog.FEHLER_BLANKO,
					FEHLER_BEIM_LADEN_DER_XML_DATEI, je);
			container = null;
		}
		if (container != null)
			container.streitgenossen = fixGesamtschuldnerschaften(container.streitgenossen);
		return container;
	}

	/**
	 * Statische Convenience-Methode, die für Programmteile ein neues
	 * Parameterobjekt für die Verfahrensdaten erzeugt, die früher
	 * VerfahrensDatenContainer genutzt haben, vor allem für den
	 * {@link XjustizContentHandler} und den {@link BaumbachAssistentDialog}.
	 * 
	 * @param klaeger
	 *            die BaumbachBeteiligtenListe mit den Klägern
	 * @param drittwiderbeklagten
	 *            die BaumbachBeteiligtenListe mit den Drittwiderbeklagten
	 * @param beklagten
	 *            die BaumbachBeteiligtenListe mit den Beklagten
	 * @param streitwert
	 *            den Streitwert als double
	 * @param aktenzeichen
	 *            das Aktenzeichen als String
	 */
	static VerfahrensDatenContainer erstelleContainer(
			BaumbachBeteiligtenListe klaeger,
			BaumbachBeteiligtenListe drittwiderbeklagte,
			BaumbachBeteiligtenListe beklagte, double streitwert,
			String aktenzeichen) {
		VerfahrensDatenContainer c = new VerfahrensDatenContainer();
		/* Erst die BaumbachBeteiligtenListen einpacken */
		Streitgenossen s = c.streitgenossen;
		s.klaeger = klaeger;
		s.drittwiderbeklagte = drittwiderbeklagte;
		s.beklagte = beklagte;
		/*
		 * Vorher gab's kein Feld fürs Vorhandensein einer Widerklage, jetzt
		 * gibt's eins, dann muss es auch richtig gefüllt werden. Die Methode
		 * bleibt dieselbe.
		 */
		s.widerklage = gibtsDrittwiderbeklagte(c)
				|| (gibtsKlaeger(c) && s.klaeger
						.enthaeltAnWiderklageBeteiligte())
				|| (gibtsBeklagte(c) && s.beklagte
						.enthaeltAnWiderklageBeteiligte());
		/* Dann die allgemeinen Teile einpacken */
		Allgemein a = c.allgemein;
		a.speicherdatum = speicherDatum();
		a.aktenzeichen = aktenzeichen;
		a.streitwert = streitwert;
		/* Damit der Rest nicht leer ist */
		c = setzeAlleStreitwerte(setzeAlleBeteiligte(c));
		return c;
	}

	/**
	 * Die Methode dient dazu, alle Streitwertangaben auf den im
	 * {@link VerfahrensDatenContainer} angegebenen höchsten Streitwert der
	 * Streitgenossen zu setzen.
	 * 
	 * @param container
	 *            ein {@link VerfahrensDatenContainer}
	 * @return den "aufgefüllten" {@link VerfahrensDatenContainer}
	 */
	private static VerfahrensDatenContainer setzeAlleStreitwerte(
			VerfahrensDatenContainer container) {
		final double streitwert = container.allgemein.streitwert;
		if (streitwert <= 0)
			return container;
		/* Zuerst auf Registerkarte 1 verteilen */
		Teilklageruecknahme t = container.teilklageruecknahme;
		t.streitwerteUndObsiegen = new ArrayList<Long>();
		t.streitwerteUndObsiegen.add((long) streitwert);
		t.streitwerteUndObsiegen.add((long) streitwert);
		t.streitwerteUndObsiegen.add((long) streitwert);
		/* Dann auf Registerkarte 4 */
		container.gebuehrenBerechnung.streitwert = (long) streitwert;
		/* und Rückgabe */
		return container;
	}

	/**
	 * Die Methode dient dazu, die Beteiligten hinsichtlich genusNumerus anhand
	 * des Inhalts der {@link BaumbachBeteiligtenListe
	 * BaumbachBeteiligtenListen} der Streitgenossen zu setzen.
	 * 
	 * @param container
	 *            ein {@link VerfahrensDatenContainer}
	 * @return den "aufgefüllten" {@link VerfahrensDatenContainer}
	 */
	private static VerfahrensDatenContainer setzeAlleBeteiligte(
			VerfahrensDatenContainer container) {
		/* Erst den Genus/Numerus zu ermitteln */
		BaumbachBeteiligtenListe klaegerseite = new BaumbachBeteiligtenListe();
		klaegerseite.addAll(container.streitgenossen.klaeger);
		klaegerseite.addAll(container.streitgenossen.drittwiderbeklagte);
		Beteiligter kl = ermittleGenusNumerus(klaegerseite, Beteiligter.KLAEGER);
		BaumbachBeteiligtenListe beklagtenseite = new BaumbachBeteiligtenListe();
		beklagtenseite.addAll(container.streitgenossen.beklagte);
		Beteiligter bk = ermittleGenusNumerus(beklagtenseite,
				Beteiligter.BEKLAGTE);
		ArrayList<Beteiligter> parteien = new ArrayList<Beteiligter>();
		/* Dann das Ergebnis einsetzen */
		parteien.add(kl);
		parteien.add(bk);
		container.teilklageruecknahme.parteien = parteien;
		container.staffelung.parteien = parteien;
		/* Und zurückgeben */
		return container;
	}

	/**
	 * Die Methode dient dazu, den Genus/Numerus zu ermitteln
	 * 
	 * @param liste
	 *            eine {@link BaumbachBeteiligtenListe}
	 * @param typ
	 *            {@link Beteiligter#KLAEGER} oder {@link Beteiligter#BEKLAGTE}
	 * @return einen {@link Beteiligter Beteiligten} mit dem richtigen
	 *         genusNumerus: Wenn die jeweilige Liste mehrere Beteiligte hat,
	 *         ist Plural angesagt. Wenn männliche Beteiligte in der Liste sind,
	 *         dann ist die Liste grammatikalisch männlich; wenn's nur weibliche
	 *         sind, dann weiblich
	 */
	private static Beteiligter ermittleGenusNumerus(
			BaumbachBeteiligtenListe liste, int typ) {
		Beteiligter bt = new Beteiligter(typ, Beteiligter.MAENNLICH);
		if (liste.size() > 0 && liste.size() < 2)
			bt.setGenusNumerus(liste.getGenusNumerusFor(0));
		else {
			int genus = Beteiligter.PLURAL;
			boolean maenner = false;
			for (BaumbachBeteiligter bb : liste)
				if (bb.getGenusNumerus() == Beteiligter.MAENNLICH)
					maenner = true;
			if (!maenner)
				genus += Beteiligter.WEIBLICH;
			bt.setGenusNumerus(genus);
			/*
			 * die laufende Nummer entspricht bei den Auswahllisten an der
			 * Oberfläche dem genusNumerus
			 */
			bt.setLfdNr(genus);
		}
		return bt;
	}

	/**
	 * @param container
	 *            der zu prüfende {@link VerfahrensDatenContainer}
	 * @return <code>true</code>, wenn der {@link VerfahrensDatenContainer} eine
	 *         nicht-leere Klägerliste enthält.
	 */
	static boolean gibtsKlaeger(VerfahrensDatenContainer container) {
		return container.streitgenossen.klaeger != null
				&& !container.streitgenossen.klaeger.isEmpty();
	}

	/**
	 * @param container
	 *            der zu prüfende {@link VerfahrensDatenContainer}
	 * @return <code>true</code>, wenn es darin eine nicht-leere
	 *         Drittwiderbeklagten-Liste gibt
	 */
	static boolean gibtsDrittwiderbeklagte(VerfahrensDatenContainer container) {
		return container.streitgenossen.drittwiderbeklagte != null
				&& !container.streitgenossen.drittwiderbeklagte.isEmpty();
	}

	/**
	 * @param container
	 *            der zu prüfende {@link VerfahrensDatenContainer}
	 * @return <code>true</code>, wenn es darin eine nicht-leere Beklagten-Liste
	 *         gibt
	 */
	static boolean gibtsBeklagte(VerfahrensDatenContainer container) {
		return container.streitgenossen.beklagte != null
				&& !container.streitgenossen.beklagte.isEmpty();
	}

	// private static KostenTenorContainer erstelleContainer() {
	// KostenTenorContainer container = new KostenTenorContainer();
	// Beteiligter kl = new Beteiligter(Beteiligter.KLAEGER,
	// Beteiligter.MAENNLICH);
	// Beteiligter bk = new Beteiligter(Beteiligter.BEKLAGTE,
	// Beteiligter.WEIBLICH);
	//
	// /*
	// * Allgemeine Programmeinstellungen, die nicht vom Container selbst
	// * gesetzt werden
	// */
	// container.allgemein.b92ii = true;
	//
	// /* Erste Registerkarte Teilklagerücknahme */
	// container.teilklageruecknahme.parteien.add(kl);
	// container.teilklageruecknahme.parteien.add(bk);
	// container.teilklageruecknahme.streitwerteUndObsiegen.add(10000L);
	// container.teilklageruecknahme.streitwerteUndObsiegen.add(5000L);
	// container.teilklageruecknahme.streitwerteUndObsiegen.add(5000L);
	// container.teilklageruecknahme.vorher = new GebuehrenTatbestand[] {
	// GerichtsGebuehrenTabelle.KV1210,
	// AnwaltsGebuehrenTabelle.VV3100, AnwaltsGebuehrenTabelle.VV3100, };
	// container.teilklageruecknahme.nachher = new GebuehrenTatbestand[] {
	// AnwaltsGebuehrenTabelle.VV3104, AnwaltsGebuehrenTabelle.VV3104, };
	//
	// /* Zweite Registerkarte Streitgenossen */
	// container.streitgenossen.widerklage = false;
	// container.streitgenossen.wirtschaftlichIdentisch = false;
	// container.streitgenossen.klaeger = new BaumbachBeteiligtenListe();
	// container.streitgenossen.klaeger.add(new BaumbachBeteiligter(
	// Beteiligter.KLAEGER, Beteiligter.MAENNLICH, 0.0, 0.0, false));
	// container.streitgenossen.drittwiderbeklagte = new
	// BaumbachBeteiligtenListe();
	// container.streitgenossen.beklagte = new BaumbachBeteiligtenListe();
	// container.streitgenossen.beklagte.add(new BaumbachBeteiligter(
	// Beteiligter.BEKLAGTE, Beteiligter.WEIBLICH, 1200, 1200, false));
	// container.streitgenossen.beklagte.add(new BaumbachBeteiligter(
	// Beteiligter.BEKLAGTE, Beteiligter.WEIBLICH, 1200, 0, false));
	// container.streitgenossen.beklagte
	// .add(new BaumbachGesamtschuldnerschaft(
	// Beteiligter.BEKLAGTE,
	// Beteiligter.MAENNLICH,
	// 0,
	// false,
	// new int[] { 0, 1 },
	// (BaumbachBeteiligtenListe) container.streitgenossen.beklagte));
	//
	// /* Dritte Registerkarte Forderungsstaffelung */
	// container.staffelung.parteien.add(kl);
	// container.staffelung.parteien.add(bk);
	// container.staffelung.forderungen
	// .add(new KlageForderung(10000.0, 7500.0));
	// container.staffelung.forderungen.add(new AufrechnungsForderung(7500.0,
	// 5000.0));
	// container.staffelung.forderungen.add(new AufrechnungsForderung(2500.0,
	// 1000.0));
	//
	// /* Vierte Registerkarte Freie Gebührenberechnung */
	// container.gebuehrenBerechnung.streitwert = 10000L;
	// container.gebuehrenBerechnung.gebuehren = new GebuehrenTatbestand[] {
	// GerichtsGebuehrenTabelle.KV1210,
	// AnwaltsGebuehrenTabelle.VV3100, AnwaltsGebuehrenTabelle.VV1008,
	// AnwaltsGebuehrenTabelle.VV3104,
	// GerichtsGebuehrenTabelle.KV9003,
	// AnwaltsGebuehrenTabelle.VV7008_R,
	// GerichtsGebuehrenTabelle.KV9003 };
	//
	// return container;
	// }

	// public static void main(String[] arg) throws JAXBException {
	// KostenTenorXmlLadenSpeichern db = new KostenTenorXmlLadenSpeichern();
	// KostenTenorContainer container = erstelleContainer();
	// db.speicherXmlDialog(null, container);
	// db.ladeXmlDialog(null);
	// }
}
