/**
 * GebuehrenBerechnungsAufstellung.java
 * eu.gronos.kostenrechner (Kostenrechner)
 */
package eu.gronos.kostenrechner;

import java.util.ArrayList;

/**
 * Klasse zur Berechnung von Gebühren
 *
 * @author Peter Schuster (setrok)
 * @date 24.08.2014
 *
 */
class GebuehrenBerechnungsAufstellung implements Tenorierend {

	private static final String BESCHREIBUNG = "Berechnung von Gebühren";
	private final ArrayList<GebuehrenTatbestand> values;
	private final long streitwert;

	/**
	 * Der einzige Konstruktor.
	 * 
	 * @param values
	 *            ArrayList&lt;GebuehrenTatbestand&gt; mit den
	 *            GebuehrenSatzTatbeständen, AuslagenTatbeständen und
	 *            MehrwertsteuerTatbeständen; darf nicht null sein.
	 * @param streitwert
	 *            ein long mit dem Streitwert, muss größer/gleich 0 sein.
	 * @throws NullPointerException
	 *             wenn values null ist
	 * @throws IllegalArgumentException
	 *             wenn streitwert kleiner 0 ist
	 */
	GebuehrenBerechnungsAufstellung(ArrayList<GebuehrenTatbestand> values,
			long streitwert) throws NullPointerException,
			IllegalArgumentException {
		super();
		if (values != null)
			this.values = values;
		else
			throw new NullPointerException(
					"Die Liste der Tatbestände darf nicht leer (null)");
		if (streitwert < 0)
			throw new IllegalArgumentException(
					"Der Streitwert muss größer/gleich 0 sein");
		else
			this.streitwert = streitwert;
	}

	@Override
	public String erzeugeTenor() {
		return berrechneGebuehrenAufstellung();
	}

	@Override
	public String getTenorBeschreibung() {
		return BESCHREIBUNG;
	}

	/**
	 * Die Methode soll die Summe aller Gebühren der Tabelle liefern.
	 * 
	 * @return einen String mit der Aufstellung aller GebührenTatbestände und
	 *         ihres Betrags für diesen Streitwert
	 */
	String berrechneGebuehrenAufstellung() {
		StringBuffer aufstellung = new StringBuffer(
				"Die Gebühren berechnen sich wie folgt:\n");
		final String zeilenFormat = "%s \t %s \t %,.2f EUR;%n";
		double summe = 0;
		double rvgsumme = 0;
		for (GebuehrenTatbestand gt : values) {
			/*
			 * errechneGebuehr ignoriert MehrwertsteuerTatbestände; diese muss
			 * deshalb extra gerechnet werden.
			 */
			if (gt instanceof MehrwertsteuerTatbestand) {
				final MehrwertsteuerTatbestand mwst = (MehrwertsteuerTatbestand) gt;
				final double einzelposition = mwst.errechneSteuer(rvgsumme);
				final String bemerkung = String.format("(%,.1f%%)",
						mwst.getSteuerSatz() * 100.0);
				summe += einzelposition;
				aufstellung.append(String.format(zeilenFormat,
						gt.getBezeichnung(), bemerkung, einzelposition));
				/*
				 * Wenn die Gebühren isoliert werden, die RVG-Summe wieder auf 0
				 * setzen.
				 */
				if (mwst.isIsoliert())
					rvgsumme = 0.0;
			} else {
				/* summe um die Gebühr erhöhen */
				final double einzelposition = StreitwertReduktion
						.errechneGebuehr(gt, streitwert);
				String ergaenzung = "";
				String bemerkung = "";
				if (gt instanceof GebuehrenErhoehungsTatbestand)
					ergaenzung = String.format(" x %d",
							((GebuehrenErhoehungsTatbestand) gt).getAnzahl());
				if (gt instanceof GebuehrenSatzTatbestand)
					bemerkung = String.format("(%,.1f)",
							((GebuehrenSatzTatbestand) gt).getSatz());
				summe += einzelposition;
				aufstellung.append(String.format(zeilenFormat,
						gt.getBezeichnung() + ergaenzung, bemerkung,
						einzelposition));
				/*
				 * rvgsumme nur erhöhen, wenn RVG-Gebühr (wird für MwSt
				 * gebraucht).
				 */
				if (gt.getGebuehrenKlasse() == AnwaltsGebuehrenTabelle.class)
					rvgsumme += StreitwertReduktion.errechneGebuehr(gt,
							streitwert);
			}
		}
		aufstellung.append(String.format(zeilenFormat, "", "Gesamt", summe));
		return aufstellung.toString();
	}

}
