/**
 * GebuehrenTatbestandTabelle.java
 * eu.gronos.kostenrechner (Kostenrechner)
 */
package eu.gronos.kostenrechner;

import java.util.ArrayList;

import javax.swing.table.AbstractTableModel;

/**
 * Ein AbstractTableModel für Gebührentatbestände, die intern als ArrayList
 * verwaltet werden.
 * 
 * @author Peter Schuster (setrok)
 * @date: 04.05.2014
 *
 * @todo TODO Reihenfolge muss sich ändern lassen
 */
class GebuehrenTatbestandTableModel extends AbstractTableModel implements
		RowHandler<GebuehrenTatbestand>, RowList<GebuehrenTatbestand>,
		TooltipLieferant {

	private static final int COLUMN_COUNT = 3;
	private static final long serialVersionUID = 2980582500842074125L;
	private static final int SPALTE_BEZEICHNUNG = 0;
	private static final int SPALTE_SATZ = 1;
	private static final int SPALTE_VERZEICHNIS = 2;
	private static final Class<?>[] COLUMN_CLASSES = new Class<?>[] {
			String.class, Double.class, String.class };
	// private GebuehrenTatbestand[] values;
	private ArrayList<GebuehrenTatbestand> values;
	private static final String[] COLUMN_NAMES = new String[] {
			"Gebührentatbestand", "Satz", "Verzeichnis" };
	private static final String[] COLUMN_TOOLTIPS = new String[] {
			"Der Gebührentatbestand aus dem Kostenverzeichnis bzw. Vergütungsverzeichnis",
			"Der Gebührensatz bzw. die Höhe der Auslagen",
			"Das Kostenverzeichnis bzw. Vergütungsverzeichnis" };

	/**
	 * Baut das AbstractTableModel mit den Werten der Gebührentatbestände auf.
	 * 
	 * @param values
	 *            ein eindimensionales Array GebuehrenTatbestand[]
	 * 
	 */
	public GebuehrenTatbestandTableModel(GebuehrenTatbestand[] values) {
		super();
		// this.values = values;
		this.values = new ArrayList<GebuehrenTatbestand>();
		for (GebuehrenTatbestand value : values)
			this.values.add(value);
	}

	/**
	 * Returns the number of rows in the model. A JTable uses this method to
	 * determine how many rows it should display. This method should be quick,
	 * as it is called frequently during rendering.
	 * 
	 * @return the number of rows in the model
	 * 
	 * @see javax.swing.table.TableModel#getRowCount()
	 */
	@Override
	public int getRowCount() {
		if (values != null)
			return values.size();
		else
			return 0;
	}

	/**
	 * Returns the number of columns in the model. A JTable uses this method to
	 * determine how many columns it should create and display by default.
	 * 
	 * @return the number of columns in the model
	 * 
	 * @see javax.swing.table.TableModel#getColumnCount()
	 */
	@Override
	public int getColumnCount() {
		return COLUMN_COUNT;
	}

	/**
	 * Die Methode getValueAt dient dazu, den Wert einer bestimmten Zelle
	 * zurückzugeben.
	 * 
	 * @param rowIndex
	 *            die gewünschte Zeile
	 * @param columnIndex
	 *            die gewünschte Spalte, wobei 0 die Bezeichnung als String, 1
	 *            der Gebührensatz als Double und 2 die Gebührentabelle (also KV
	 *            GKG oder VV RVG) als String ist.
	 * @return den Zellinhalt
	 * 
	 * @see javax.swing.table.TableModel#getValueAt(int, int)
	 */
	@Override
	public Object getValueAt(int rowIndex, int columnIndex) {
		switch (columnIndex) {
		case SPALTE_BEZEICHNUNG:
			return getBezeichnungAt(rowIndex);
		case SPALTE_SATZ:
			return getSatzAt(rowIndex);
		case SPALTE_VERZEICHNIS:
			return getVerzeichnisAt(rowIndex);
		default:
			return null;
		}
	}

	/**
	 * Die Methode dient dazu, von getValueAt() aufgerufen zu werden.
	 * 
	 * @param rowIndex
	 *            die gewünschte Zeile
	 * @return "KV GKG" oder "VV RVG" als String
	 */
	private String getVerzeichnisAt(int rowIndex) {
		if (values == null)
			return null;
		if (GerichtsGebuehrenTabelle.class == values.get(rowIndex)
				.getGebuehrenKlasse())
			return "KV GKG";
		else if (AnwaltsGebuehrenTabelle.class == values.get(rowIndex)
				.getGebuehrenKlasse())
			return "VV RVG";
		else
			return null;
	}

	/**
	 * Die Methode dient dazu, von getValueAt() aufgerufen zu werden.
	 * 
	 * @param rowIndex
	 *            die gewünschte Zeile
	 * @return den satz eines GebuehrenTatbestand bzw. die auslagen eines
	 *         AuslagenTatbestands
	 */
	private Double getSatzAt(int rowIndex) {
		if (values != null) {
			GebuehrenTatbestand gt = values.get(rowIndex);
			if (gt instanceof AuslagenTatbestand)
				return ((AuslagenTatbestand) gt).getBetrag();
			else if (gt instanceof GebuehrenSatzTatbestand)
				return ((GebuehrenSatzTatbestand) gt).getSatz();
			else if (gt instanceof MehrwertsteuerTatbestand)
				return ((MehrwertsteuerTatbestand) gt).getSteuerSatz();
		}
		return null;
	}

	/**
	 * Die Methode dient dazu, von getValueAt() aufgerufen zu werden.
	 * 
	 * @param rowIndex
	 *            die gewünschte Zeile
	 * @return die {@link GebuehrenSatzTatbestand#getBezeichnung() Bezeichnung}
	 *         des Tatbestands in der angegebenen Zeile
	 */
	private String getBezeichnungAt(int rowIndex) {
		if (values != null)
			return values.get(rowIndex).getBezeichnung();
		return null;
	}

	/**
	 * Die Methode gibt die Spaltenüberschrift der Spalte zurück
	 * 
	 * @param columnIndex
	 *            die gewünschte Spalte
	 * @return die Spaltenüberschrift der Spalte
	 * 
	 * @see javax.swing.table.AbstractTableModel#getColumnName(int)
	 */
	@Override
	public String getColumnName(int columnIndex) {
		if (columnIndex < COLUMN_COUNT)
			return COLUMN_NAMES[columnIndex];
		else
			return super.getColumnName(columnIndex);
	}

	/**
	 * Die Methode getColumnClass dient dazu, die Klasse des Inhalts der
	 * abgefragten Spalte zurück zu geben.
	 * 
	 * @param columnIndex
	 *            die abgefragte Spalte
	 * @return die Class, die zu der Spalte passt
	 * 
	 * @see javax.swing.table.AbstractTableModel#getColumnClass(int)
	 */
	@Override
	public Class<?> getColumnClass(int columnIndex) {
		if (columnIndex < COLUMN_COUNT)
			return COLUMN_CLASSES[columnIndex];
		else
			return super.getColumnClass(columnIndex);

	}

	/**
	 * Die Methode dient dazu, den ganzen Tabelleninhalt zurückzugeben.
	 * 
	 * @return ein Array GebuehrenTatbestand[] mit allen Werten der Tabelle
	 *
	 * @see eu.gronos.kostenrechner.RowList#toArray()
	 * @see java.util.ArrayList#toArray(T[])
	 */
	@Override
	public GebuehrenTatbestand[] toArray() {
		if (values != null) {
			GebuehrenTatbestand[] array = new GebuehrenTatbestand[values.size()];
			// for (int row = 0; row < getRowCount(); row++)
			// array[row] = values.get(row);
			array = values.toArray(array);
			return array;
		} else
			return null;
	}

	/**
	 * Die Methode getAllValues dient dazu, den ganzen Tabelleninhalt
	 * zurückzugeben.
	 * 
	 * @return eine ArrayList<GebuehrenTatbestand> mit allen Werten der Tabelle
	 * 
	 * @see eu.gronos.kostenrechner.RowList#getAllValues()
	 */
	@Override
	public ArrayList<GebuehrenTatbestand> getAllValues() {
		return values;
	}

	/**
	 * Entfernt alle Werte aus der zugrundeliegenden ArrayList oder legt eine
	 * solche an, falls noch nicht vorhanden.
	 * 
	 * @see eu.gronos.kostenrechner.RowList#clear()
	 */
	@Override
	public void clear() {
		if (values == null)
			this.values = new ArrayList<GebuehrenTatbestand>();
		else
			values.clear();
		fireTableDataChanged();
	}

	/**
	 * Hängt alle Werte des übergebenen Arrays der zugrundeliegenden ArrayList
	 * hinzu.
	 * 
	 * @param c
	 *            alle hinzuzufügenden Werte als GebuehrenTatbestand[]
	 * @throws NullPointerException
	 *             wenn <code>array</code> leer ist.
	 * @throws UnsupportedOperationException
	 *             wird nicht geworfen, weil diese TableModel die Operation
	 *             unterstützt.
	 * 
	 * @see eu.gronos.kostenrechner.RowList#addAll(java.lang.Object[])
	 */
	@Override
	public void addAll(GebuehrenTatbestand[] c) throws NullPointerException,
			UnsupportedOperationException {
		if (c == null)
			throw new NullPointerException("Das array darf nicht null sein!");
		for (GebuehrenTatbestand gt : c)
			values.add(gt);
		fireTableDataChanged();
	}

	/**
	 * Hängt einen neuen GebuehrenTatbestand hinten an die values an.
	 * 
	 * @param gt
	 *            der anzuhängende GebuehrenTatbestand
	 *
	 * @see eu.gronos.kostenrechner.RowHandler#addRow(java.lang.Object)
	 */
	@Override
	public void addRow(GebuehrenTatbestand gt) throws IllegalArgumentException {
		if (gt != null) {
			values.add(gt);
			fireTableRowsInserted(values.size() - 1, values.size() - 1);
		}
	}

	/**
	 * Fügt einen neuen GebuehrenTatbestand an der gewünschten Stelle den values
	 * zu. Vorhandene Zeilen werden nach hinten verschoben.
	 * 
	 * @param rowIndex
	 *            index at which the specified row is to be inserted
	 * @param gt
	 *            der anzuhängende GebuehrenTatbestand
	 * @throws IllegalArgumentException
	 * @see java.util.ArrayList#add(int, Object)
	 */
	public void addRowAt(int rowIndex, GebuehrenTatbestand gt)
			throws IllegalArgumentException {
		if (gt != null && rowIndex > -1 && rowIndex <= values.size()) {
			values.add(rowIndex, gt);
			fireTableRowsInserted(rowIndex, rowIndex);
		}
	}

	/**
	 * Entfernt eine Zeile aus der Tabelle
	 * 
	 * @param rowIndex
	 *            die zu entfernende Zeile
	 * @throws IndexOutOfBoundsException
	 * @see eu.gronos.kostenrechner.RowHandler#removeRow(int)
	 */
	@Override
	public void removeRow(int rowIndex) throws IndexOutOfBoundsException,
			IllegalArgumentException {
		if (values != null && rowIndex > -1 && rowIndex < getRowCount())
			values.remove(rowIndex);
		fireTableRowsDeleted(rowIndex, rowIndex);
	}

	/**
	 * Gibt eine Zeile als GebuehrenTatbestand zurück
	 * 
	 * @param rowIndex
	 *            der Index der Zeile
	 * @return den GebuehrenTatbestand für die Zeile
	 * 
	 * @see eu.gronos.kostenrechner.RowHandler#getRow(int)
	 */
	@Override
	public GebuehrenTatbestand getRow(int rowIndex)
			throws IndexOutOfBoundsException {
		return values.get(rowIndex);
	}

	/**
	 * Die Methode gibt den Tooltip-Text für die angegebene Spalte zurück.
	 * 
	 * @param columnIndex
	 *            der Index der Zeile
	 * @return den Tooltip-Text für die Zeile
	 * 
	 * @see eu.gronos.kostenrechner.TooltipLieferant#getTooltipText(int)
	 */
	@Override
	public String getTooltipText(int columnIndex) {
		return COLUMN_TOOLTIPS[columnIndex];
	}

	/**
	 * Die Methode soll die Summe aller Gebühren der Tabelle liefern.
	 * 
	 * @param streitwert
	 *            ein long mit dem Streitwert
	 * @return ein double mit der Summe der Gebühren für diesen Streitwert
	 */
	double errechneGebuehrenSumme(long streitwert) {
		double summe = 0;
		double rvgsumme = 0;
		for (GebuehrenTatbestand gt : values) {
			/*
			 * errechneGebuehr ignoriert MehrwertsteuerTatbestände; diese muss
			 * deshalb extra gerechnet werden.
			 */
			if (gt instanceof MehrwertsteuerTatbestand) {
				MehrwertsteuerTatbestand mwst = (MehrwertsteuerTatbestand) gt;
				summe += mwst.errechneSteuer(rvgsumme);
				/*
				 * Wenn die Gebühren isoliert werden, die RVG-Summe wieder auf 0
				 * setzen.
				 */
				if (mwst.isIsoliert())
					rvgsumme = 0.0;
			} else {
				/* summe um die Gebühr erhöhen */
				summe += StreitwertReduktion.errechneGebuehr(gt, streitwert);
				/*
				 * rvgsumme nur erhöhen, wenn RVG-Gebühr (wird für MwSt
				 * gebraucht).
				 */
				if (gt.getGebuehrenKlasse() == AnwaltsGebuehrenTabelle.class)
					rvgsumme += StreitwertReduktion.errechneGebuehr(gt,
							streitwert);
			}
		}
		return summe;
	}
}
