package eu.gronos.kostenrechner;

import javax.swing.table.AbstractTableModel;

class ScalableBeteiligtenTableModel extends
		AbstractTableModel {

	private static final long serialVersionUID = -255054438294776137L;
	private Object[][] values;
	private int columnCount;
	private final static Class<?>[] COLUMN_CLASSES = new Class<?>[] {
			Integer.class, Beteiligter.class, Double.class, Double.class,
			Boolean.class };
	private final static String[] COLUMN_NAMES = new String[] { "#",
			"Parteibezeichnung (m/w)", "Streitwert", "Unterliegen",
			"Widerklage?" };

	/**
	 * Konstruktor für die unterste Ausbaustufe der Tabelle. Nimmt die
	 * values als Inhalt auf und füllt sie ggfs. mit neuen Beteiligten des
	 * übergebenen Beteiligtentyps
	 * 
	 * @param values
	 *            ein Array Object[][]
	 * @param beteiligtenTyp
	 *            eine der Konstanten Beteiligter.KLAEGER,
	 *            Beteiligter.DRITTWIDERBEKLAGTE, Beteiligter.BEKLAGTE als
	 *            Standardwert.
	 */
	protected ScalableBeteiligtenTableModel(Object[][] values,
			int beteiligtenTyp) {
		columnCount = 2;
		this.values = values;
		if (values == null)
			return;
		else
			for (int rowIndex = 0; rowIndex < values.length; rowIndex++)
				if (this.values[rowIndex][0] == null) {
					this.values[rowIndex][0] = new Beteiligter(
							beteiligtenTyp, Beteiligter.MAENNLICH);
				}

	}

	/**
	 * Konstruktor für die zweite Ausbaustufe der Tabelle. Nimmt die values
	 * als Inhalt auf und füllt sie ggfs. mit neuen Beteiligten des
	 * übergebenen Beteiligtentyps und Streitwerts
	 * 
	 * @param values
	 *            ein Array Object[][]
	 * @param beteiligtenTyp
	 *            eine der Konstanten Beteiligter.KLAEGER,
	 *            Beteiligter.DRITTWIDERBEKLAGTE, Beteiligter.BEKLAGTE als
	 *            Standardwert.
	 * @param streitwert
	 *            der Standardwert für den Streitwert
	 */
	public ScalableBeteiligtenTableModel(Object[][] values,
			int beteiligtenTyp, double streitwert) {
		this(values, beteiligtenTyp);
		columnCount = 3;
		setEachStreitwert(streitwert, false);
	}

	/**
	 * Konstruktor für die dritte (bisher höchste) Ausbaustufe der Tabelle.
	 * Nimmt die values als Inhalt auf und füllt sie ggfs. mit neuen
	 * Beteiligten des übergebenen Beteiligtentyps und Streitwerts
	 * 
	 * @param values
	 *            ein Array Object[][]
	 * @param beteiligtenTyp
	 *            eine der Konstanten Beteiligter.KLAEGER,
	 *            Beteiligter.DRITTWIDERBEKLAGTE, Beteiligter.BEKLAGTE als
	 *            Standardwert.
	 * @param streitwert
	 *            der Standardwert für den Streitwert
	 * @param unterliegen
	 *            der Standardwert für das Unterliegen
	 */
	public ScalableBeteiligtenTableModel(Object[][] values,
			int beteiligtenTyp, double streitwert, double unterliegen) {
		this(values, beteiligtenTyp, streitwert);
		columnCount = 4;
		for (int rowIndex = 0; rowIndex < values.length; rowIndex++)
			if (this.values[rowIndex][2] == null) {
				this.values[rowIndex][2] = new Double(unterliegen);
			}
	}

	/**
	 * Setzt den Streitwert für jedes einzelne Element
	 * 
	 * @param streitwert
	 *            den zu setzenden Streitwert als double
	 * @param ueberschreiben
	 *            false = der Streitwert wird nur gesetzt, wenn bisher null;
	 *            true = Streitwert immer setzen
	 */
	void setEachStreitwert(double streitwert, boolean ueberschreiben) {
		for (int rowIndex = 0; rowIndex < this.values.length; rowIndex++)
			if (this.values[rowIndex][1] == null || ueberschreiben) {
				this.values[rowIndex][1] = new Double(streitwert);
			}
		fireTableRowsUpdated(0, getRowCount() - 1);
	}

	/**
	 * Setzt für alle Beteiligte der Tabelle einheitlich, ob sie an der
	 * Widerklage beteiligt sind.
	 * 
	 * @param anWiderklageBeteiligt
	 *            der zu setzende Boolesche Wahrheitswert
	 * @param ueberschreiben
	 *            false = wird nur gesetzt, wenn bisher null; true = immer
	 *            setzen
	 */
	void setEachAnwiderklagebeteiligt(boolean anWiderklageBeteiligt,
			boolean ueberschreiben) {
		for (int rowIndex = 0; rowIndex < this.values.length; rowIndex++)
			if (this.values[rowIndex][3] == null || ueberschreiben) {
				this.values[rowIndex][3] = new Boolean(
						anWiderklageBeteiligt);
			}
	}

	/**
	 * Für Gebrauch mit JList. Gibt die Beteiligten-Spalte zurück.
	 * 
	 * @return die Spalte mit den Beteiligten als eindimennsionales Array
	 *         Beteiligter[]
	 */
	Beteiligter[] getBeteiligterColumn() {
		Beteiligter[] bba = new Beteiligter[getRowCount()];
		for (int row = 0; row < getRowCount(); row++)
			bba[row] = (Beteiligter) getValueAt(row, 1);
		return bba;
	}

	/**
	 * @see javax.swing.table.AbstractTableModel#getColumnName(int)
	 */
	@Override
	public String getColumnName(int column) {
		return COLUMN_NAMES[column];
	}

	/**
	 * @see javax.swing.table.AbstractTableModel#getColumnClass(int)
	 */
	@Override
	public Class<?> getColumnClass(int columnIndex) {
		return COLUMN_CLASSES[columnIndex];
	}

	/**
	 * Die erste Spalte ist nie veränderlich, sonst ist immer nur die letzte
	 * Spalte beschreibbar
	 * 
	 * @see javax.swing.table.AbstractTableModel#isCellEditable(int, int)
	 */
	@Override
	public boolean isCellEditable(int rowIndex, int columnIndex) {
		return values != null && values.length > rowIndex
				&& columnCount - 1 == columnIndex;
	}

	/**
	 * @see javax.swing.table.AbstractTableModel#setValueAt(java.lang.Object,
	 *      int, int)
	 */
	@Override
	public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
		if (values == null || values.length <= rowIndex || columnIndex == 0
				|| columnCount <= columnIndex)
			return;
		else {
			values[rowIndex][columnIndex - 1] = aValue;
			super.fireTableCellUpdated(rowIndex, columnIndex);
		}
	}

	/**
	 * @see javax.swing.table.TableModel#getRowCount()
	 */
	@Override
	public int getRowCount() {
		return values == null ? 0 : values.length;
	}

	/**
	 * @see javax.swing.table.TableModel#getColumnCount()
	 */
	@Override
	public int getColumnCount() {
		return this.columnCount;
	}

	/**
	 * @see javax.swing.table.TableModel#getValueAt(int, int)
	 */
	@Override
	public Object getValueAt(int rowIndex, int columnIndex) {
		// Für die erste Spalte eine Nummerierung ausgeben
		if (columnIndex == 0)
			return rowIndex + 1;
		else
			// sonst den passenden Wert aus values (MERKER: values hat eine
			// Spalte weniger als die Tabelle selbst!)
			return (values == null || values.length <= rowIndex || columnCount <= columnIndex) ? null
					: values[rowIndex][columnIndex - 1];
	}

	/**
	 * Die Methode gibt alle Werte zurück
	 * 
	 * @return alle Werte der Tabelle
	 */
	Object[][] getValues() {
		return values;
	}

	double getHoechstenStreitwert() {
		double sw = 0;
		for (int row = 0; row < getRowCount(); row++) {
			Double dd = (Double) getValueAt(row, 2);
			double d = dd != null ? dd.doubleValue() : 0;
			if (d > sw)
				sw = d;
		}
		return sw;
	}

}